/**
 *  EditVariableDialog.java
 *
 *  Original version by  Sreekanth Nagarajan, rewritten
 *  by Fabio Cozman.
 *
 */

package JavaBayesInterface;

import InferenceGraphs.*;

import java.awt.*;
import java.util.*;

class EditVariableDialog extends Dialog {
    NetworkPanel npan;
    InferenceGraph ig;
    InferenceGraphNode node;

    int displayed_variable_property_index, displayed_function_property_index;
    int number_extreme_points;

    Panel np, nvp, tp, ttp, vpp, fpp, npp, cbp, pp, gnp, gncp, okp, qbp, qbpp;
    Label name, new_value, type;
    Label variable_properties, function_properties, local_parameter;
    TextField text_name, text_new_value, text_local_parameter;
    CheckboxGroup types, function_types;
    Checkbox chance_type, explanation_type, no_local_credal_set_type, local_credal_set_type;
    Button new_variable_property, next_variable_property;
    Button new_function_property, next_function_property;
    TextField variable_properties_text, function_properties_text;
    Button dist_button, return_button;
    
    private final static int TOP_INSET = 5;
    private final static int LEFT_INSET = 10;
    private final static int RIGHT_INSET = 10;
    private final static int BOTTOM_INSET = 0;

    private final static String name_label = "Name:";
    private final static String new_value_label = "Values:";
    private final static String type_label = "Types:";
    private final static String chance_type_label = "Chance node";
    private final static String explanation_type_label = "Explanatory node";
    private final static String no_local_credal_set_label = "Single distribution";
    private final static String local_credal_set_label = "Credal set with extreme points";
    private final static String number_extreme_points_label = "Number of extreme points:";
    private final static String variable_properties_label = "Variable properties:";
    private final static String function_properties_label = "Function properties:";
    private final static String next_property_label = "Next";
    private final static String new_property_label = "New";
    private final static String edit_function_label = "Edit function";
    private final static String return_label = "Close window";

    /* ****************************************************** *
     * Default constructor for an EditVariableDialog object.  *
     * ****************************************************** */
    public EditVariableDialog(NetworkPanel network_panel, Frame parent,
    InferenceGraph i_g, InferenceGraphNode node) {
    	super(parent, "Edit: " + node.get_name(),
              false); // This used to be true, but bug in JDK1.1.6 does not accept it.
    	this.npan = network_panel;
    	this.ig = i_g;
    	this.node = node;

    	// Compose the frame

        // Panel for name, values and type

        // Panel for the name
        np = new Panel();
        np.setLayout(new BorderLayout());
        name = new Label(name_label);
        text_name = new TextField(30);
        np.add("West", name);
        np.add("Center", text_name);

        // Panel for the values
        nvp = new Panel();
        nvp.setLayout(new BorderLayout());
        new_value = new Label(new_value_label);
        text_new_value = new TextField(60);
        nvp.add("West", new_value);
        nvp.add("Center", text_new_value);

        // Panel for the type
        tp = new Panel();
        tp.setLayout(new BorderLayout());
        type = new Label(type_label);

        ttp = new Panel();
        ttp.setLayout(new GridLayout(2, 1));
        types = new CheckboxGroup();
        chance_type = new Checkbox(chance_type_label, types, true);
        explanation_type = new Checkbox(explanation_type_label, types, false);
        ttp.add(chance_type);
        ttp.add(explanation_type);

        qbp = new Panel();
        qbp.setLayout(new GridLayout(3,1));
        function_types = new CheckboxGroup();
        no_local_credal_set_type = new Checkbox(no_local_credal_set_label, function_types, true);
        local_credal_set_type = new Checkbox(local_credal_set_label, function_types, false);

        qbpp = new Panel();
        qbpp.setLayout(new BorderLayout());
        local_parameter = new Label(number_extreme_points_label);
        text_local_parameter = new TextField(5);
        qbpp.add("West", local_parameter);
        qbpp.add("Center", text_local_parameter);

        qbp.add(no_local_credal_set_type);
        qbp.add(local_credal_set_type);
        qbp.add(qbpp);

        tp.add("North", type);
        tp.add("West", ttp);
        tp.add("East", qbp);

        // Finish panel for name, values and type
        cbp = new Panel();
    	cbp.setLayout(new BorderLayout(10,10));
        cbp.add("North", np);
        cbp.add("Center", nvp);
        cbp.add("South", tp);

        // Panel for properties (variable, function and network)
        pp = new Panel();
        pp.setLayout(new BorderLayout());

        // Variable properties
        vpp = new Panel();
        vpp.setLayout(new BorderLayout());
        variable_properties = new Label(variable_properties_label);
        next_variable_property = new Button(next_property_label);
        new_variable_property = new Button(new_property_label);
        variable_properties_text = new TextField(40);
        vpp.add("North", variable_properties);
        vpp.add("West", next_variable_property);
        vpp.add("Center", variable_properties_text);
        vpp.add("East", new_variable_property);

        // Function properties
        fpp = new Panel();
        fpp.setLayout(new BorderLayout());
        function_properties = new Label(function_properties_label);
        next_function_property = new Button(next_property_label);
        new_function_property = new Button(new_property_label);
        function_properties_text = new TextField(40);
        fpp.add("North", function_properties);
        fpp.add("West", next_function_property);
        fpp.add("Center", function_properties_text);
        fpp.add("East", new_function_property);

        // Finish panel for properties
        pp.add("North", vpp);
        pp.add("Center", fpp);

        // Return buttons
     	okp = new Panel();
	    okp.setLayout(new FlowLayout(FlowLayout.CENTER));
    	dist_button = new Button(edit_function_label);
    	okp.add(dist_button);
        return_button =	new Button(return_label);
        okp.add(return_button);
        setLayout(new BorderLayout());
    	add("North", cbp);
    	add("Center", pp);
	    add("South", okp);

	    // Pack the whole window
    	pack();

    	// Initialize values
    	fill_dialog();
    }

    /* ************************************************ *
     * Customized show method.                          *
     * ************************************************ */
    public synchronized void show() {
       	Rectangle bounds = getParent().bounds();
    	Rectangle abounds = bounds();

    	move(bounds.x + (bounds.width - abounds.width)/ 2,
    	     bounds.y + (bounds.height - abounds.height)/2);

    	super.show();
    }

    /* ************************************************** *
     * Customize insets method.                           *
     * ************************************************** */
    public Insets insets() {
        Insets ins = super.insets();
        return(new Insets(ins.top + TOP_INSET, ins.left + LEFT_INSET,
                          ins.bottom + BOTTOM_INSET, ins.right + RIGHT_INSET));
    }

    /* ************************************************** *
     * Fill the values in the dialog area.                *
     * ************************************************** */
    private void fill_dialog() {
        String values[], all_values = "";
        Vector prop;
        String property;

        // Synchronize the network if necessary.
        ig.get_bayes_net();

        // Fill name
        text_name.setText(node.get_name());

        // Fill values
        values = node.get_values();
        for (int i=0; i<values.length; i++) {
            all_values += values[i];
            if (i != (values.length - 1)) all_values += ", ";
        }
        text_new_value.setText(all_values);

        // Set types
        if (node.is_explanation())
            types.setCurrent(explanation_type);
        else
            types.setCurrent(chance_type);

        if (node.is_credal_set())
            function_types.setCurrent(local_credal_set_type);
        else
            function_types.setCurrent(no_local_credal_set_type);

        number_extreme_points = node.number_extreme_distributions();
        text_local_parameter.setText( String.valueOf(number_extreme_points) );

        // Fill variable properties
        prop = node.get_variable_properties();
        if ((prop != null) && (prop.size() > 0)) {
            property = (String)(prop.firstElement());
            variable_properties_text.setText(property);
            displayed_variable_property_index = 0;
        }
        else
            displayed_variable_property_index = -1;

        // Fill function properties
        prop = node.get_function_properties();
        if ((prop != null) && (prop.size() > 0)) {
            property = (String)(prop.firstElement());
            function_properties_text.setText(property);
            displayed_function_property_index = 0;
        }
        else
            displayed_function_property_index = -1;
     }

    /**
     * Handle the events.  
     */
    public boolean action(Event evt, Object arg) {
        Vector prop;
        String values[], property, checked_name;

        if (evt.target == return_button)
	        dispose();
	    else if (evt.target == text_name) {
	        checked_name = ig.check_name(text_name.getText());
	        if (checked_name != null)
    	        node.set_name(checked_name);
	    }
	    else if (evt.target == text_new_value) {
            values = parse_values(text_new_value.getText());
            if (values != null)
                ig.change_values(node, values);
	    }
	    else if (evt.target == chance_type) {
	        node.set_explanation(false);
	        npan.repaint();
        }
        else if (evt.target == explanation_type) {
            node.set_explanation(true);
            npan.repaint();
        }
        else if (evt.target == no_local_credal_set_type) {
            node.set_no_local_credal_set();
        }
        else if (evt.target == local_credal_set_type) {
            node.set_local_credal_set(number_extreme_points);
        }
        else if (evt.target == text_local_parameter) {
            try {
                number_extreme_points =
                    (new Integer( text_local_parameter.getText() )).intValue();
                if (node.is_credal_set())
                    node.set_local_credal_set(number_extreme_points);
            } catch (NumberFormatException ex) {  }
        }
        else if (evt.target == new_variable_property) {
            displayed_variable_property_index = -1;
            variable_properties_text.setText("");
        }
        else if (evt.target == next_variable_property) {
            prop = node.get_variable_properties();
            if ((prop != null) && (prop.size() > 0)) {
                displayed_variable_property_index++;
                if (displayed_variable_property_index >= prop.size())
                    displayed_variable_property_index = 0;
                property = (String)(prop.elementAt(displayed_variable_property_index));
                variable_properties_text.setText(property);
            }
        }
        else if (evt.target == new_function_property) {
            displayed_function_property_index = -1;
            function_properties_text.setText("");
        }
        else if (evt.target == next_function_property) {
            prop = node.get_function_properties();
            if ((prop != null) && (prop.size() > 0)) {
                displayed_function_property_index++;
                if (displayed_function_property_index >= prop.size())
                    displayed_function_property_index = 0;
                property = (String)(prop.elementAt(displayed_function_property_index));
                function_properties_text.setText(property);
            }
        }
        else if (evt.target == variable_properties_text) {
            if (displayed_variable_property_index != -1)
                node.remove_variable_property(displayed_variable_property_index);
            node.add_variable_property(variable_properties_text.getText());
            npan.repaint();
        }
        else if (evt.target == function_properties_text) {
            if (displayed_function_property_index != -1)
                node.remove_function_property(displayed_function_property_index);
            node.add_function_property(function_properties_text.getText());
        }
        else if (evt.target == dist_button) {
            npan.edit_function(node);
        }
        else return super.action(evt, arg);

    	return true;
    }

    /* ********************************************************* */
    /* Parse the values stated in the values TextField.          */
    /* ********************************************************* */
    private String[] parse_values(String all_values) {
        String token = null, delimiters = " ,\n\t\r";
        StringTokenizer st = new StringTokenizer(all_values, delimiters);
        String vals[] = new String[ st.countTokens() ];
        int i = 0;

        while (st.hasMoreTokens()) {
            vals[i] = ig.validate_value(st.nextToken());
            i++;
        }
        return(vals);
    }

}

