/**
 *  ObserveDialog.java
 *
 *  Original version by  Sreekanth Nagarajan, rewritten
 *  by Fabio Cozman.
 *
 */

package JavaBayesInterface;

import InferenceGraphs.*;

import java.awt.*;

class ObserveDialog extends Dialog {
    InferenceGraph ig;
    InferenceGraphNode node;

    boolean observed;

    Checkbox observedBox;
    List valuesList;

    /* ************************************************* *
     * Default constructor for an ObserveDialog object.  *
     * ************************************************* */
    public ObserveDialog(Frame parent,
    InferenceGraph i_g, InferenceGraphNode node) {
    	super(parent, "Set Observe Value", 
              false); // true); Bug in JDK1.1.6 does not accept true.
    	this.ig = i_g;
    	this.node = node;

    	Panel cbp = new Panel();
    	cbp.setLayout(new FlowLayout(FlowLayout.CENTER));

    	observed = node.is_observed();
    	observedBox = new Checkbox("Observed", null, observed);
    	cbp.add(observedBox);

    	Panel listp = new Panel();
	    listp.setLayout(new GridLayout(1, 1));
    	valuesList = new List(6, false);

    	String[] values = node.get_values();
    	for (int i=0; i < values.length; i++)
	        valuesList.addItem(new String(values[i]));

    	if (observed) {
    	    valuesList.select(node.get_observed_value());
        }

    	listp.add(valuesList);

    	Panel okp = new Panel();
	    okp.setLayout(new FlowLayout(FlowLayout.CENTER));
    	okp.add(new Button("Ok"));
    	okp.add(new Button("Cancel"));

        setLayout(new BorderLayout());
    	add("North", cbp);
    	add("Center", listp);
	    add("South", okp);
    	pack();
    }

    /* ************************************************** *
     * Handle the observation events.                     *
     * ************************************************** */
    public boolean action(Event evt, Object arg) {
    	if (evt.target == observedBox) {
	        observed = observedBox.getState();
            if (observed)
    	    	valuesList.select(0);	// select first value by default
    	    else	// clear any selection
	        	valuesList.deselect(valuesList.getSelectedIndex());
    	    return super.action(evt, arg);
        }
    	else if (evt.target == valuesList) {
	        if (! observed) {
		    observed = true;
    		observedBox.setState(observed);
	        }
    	    return super.action(evt, arg);
	    }
        else if (arg.equals("Ok")) {
    	    String selValue = null;
	        observed = observedBox.getState();
	        selValue = valuesList.getSelectedItem();
    	    if (observed && selValue == null) {
	            JavaBayesHelpMessages.show(JavaBayesHelpMessages.observe_error);
	            return true;		// do not close this dialog box
            }
    	    if (observed)
    	        node.set_observation_value(selValue);
	        else
	            node.clear_observation();
	        dispose();
        }
    	else if (arg.equals("Cancel"))
	        dispose();
        else return super.action(evt, arg);
    	return true;
    }

}

