package learn;

import java.awt.Point;
import java.util.Enumeration;
import java.util.Hashtable;

import org.uva.object.Agent;
import org.uva.object.PredatorPreyAgent;

public class PredatorPreyAgentState extends QState
{
	/**
	 * Store local agent by distance
	 */
	//private Hashtable distanceTable = new Hashtable();

	/**
	 * Store local agent by x,y translation (Point)
	 */
	private Hashtable translationTable = new Hashtable();

	/**
	 * Store local agent by x,y translation (Point)
	 */
	private Hashtable preyTable = new Hashtable();

	/**
	 * Store local agent by x,y translation (Point)
	 */
	private Hashtable predatorTable = new Hashtable();

	/**
	 * Generate a new state to use in the Q table
	 * @param actingAgent the agent to calculate the next move for
	 * @param agentTable
	 */
	public PredatorPreyAgentState( PredatorPreyAgent actingAgent, Hashtable agentTable, boolean flat )
	{
		Enumeration enumerate;
		int maxDistance = actingAgent.getViewDistance();
		int localDistance;
		
		for( enumerate = agentTable.keys(); enumerate.hasMoreElements(); )
		{
			Object key = enumerate.nextElement();
			Agent agent = (Agent)agentTable.get( key );
			if( agent == actingAgent || agent.isDead() )
				continue;
			// skip other types of agents
			if( agent.group == actingAgent.group 
					&& agent instanceof PredatorPreyAgent)
			{
				// store if we can see the agent
				Point translation = actingAgent.getTranslation( agent, flat ); 
				localDistance = translation.x*translation.x+translation.y*translation.y; 
				if( localDistance <= maxDistance && actingAgent.isPointVisible( translation ) )
				{
					//distanceTable.put( localDistance, agent );
					translationTable.put( translation, agent );

					switch( ((PredatorPreyAgent)agent).getAgentType() )
					{
					case PredatorPreyAgent.TYPE_PREDATOR:
						predatorTable.put( translation, agent );
						break;
					case PredatorPreyAgent.TYPE_PREY:
						preyTable.put( translation, agent );
						break;
					}
				}
			}
		}
	}
	
	public boolean equals( QState state )
	{
		if( state instanceof PredatorPreyAgentState )
		{
			return ((PredatorPreyAgentState)state).translationTable.equals( this.translationTable );
			
			/*if(((PredatorPreyAgentState)state).table.equals( this.table )) 
			{
				System.out.println( "Equal" );
				return true;
			}*/
		}
		return false;
	}
	
	public int hashCode()
	{
		return this.translationTable.hashCode();
	}
	
	/**
	 * Get the agents we can see
	 * @return
	 */
	public Hashtable getLocalAgents()
	{
		return this.translationTable;
		//return this.distanceTable;
	}
	
	public Hashtable getCaughtPreys()
	{
		Hashtable table = new Hashtable();
		Object prey;
		for( Enumeration en = predatorTable.keys(); en.hasMoreElements() ; )
		{
			Object key = en.nextElement();
			prey = preyTable.get( key );
			if( prey != null )
				table.put( key, prey );
		}
		return table;
	}
	
}

