package org.uva.control;

import java.util.Enumeration;
import java.util.Vector;

import ws.proxyImpls.IGasSensorUvmProxy;
import ws.proxyImpls.IGenericSensorProxy;
import ws.proxyImpls.IMotionDetectorProxy;
import ws.test.SensorMap;
import ws.test.SensorServiceDescription;
import ws.test.Simulator;

import com.muvium.web.services.gateway.RPCServer;

public class SensorReader {

    public static final String	SENSOR_PROXY_IP	 	= "127.0.0.1";   // Proxy
    public static final String 	SIMULATOR_PORT 			= "81";
    public static final String 	SIMULATOR_IP 			= "10.1.1.1";
    public static final String 	SIMULATOR_INTPORT 		= "90";			// Simulator interface
    public static final String 	SIMULATOR_DEV_IP 		= "10.1.1.1";
    public static final String 	REAL_SENSOR_IP 			= "10.1.1.10";
    
    public static final int		NUMBER_OF_SENSORS 		= 8;
    /**
     * The available ports for the sensors, connect the gateway to these ports
     */
    public static final int[] SENSOR_PORTS = new int[]
    {
    	12345
    	, 12346
    	, 12347
    	, 12348
    	, 12349
    	, 12350
    	, 12351
    	, 12352
    	, 12353
    	, 12354
    };

	protected SensorMap sensors;
    protected RPCServer simulatorServer;

    public SensorReader()
    {
    	sensors = new SensorMap();
    	// add the sensors
    	for( int iSensor = 0; iSensor < NUMBER_OF_SENSORS; iSensor++ )
    	{
    		sensors.addSensor(
    				new SensorServiceDescription( iSensor
    						, SENSOR_PROXY_IP 
    						, "" + SENSOR_PORTS[iSensor] 
                            , REAL_SENSOR_IP
                            , "IMotionSensor"
                            , "uvm" )
    		);
    	}
    	boolean simSensors = false;

    	if( simSensors )
    	{
            //Create the WebService enabled Simulator on 10.1.1.1 port Port80 gateway
            RPCServer sourceServer = new RPCServer(SIMULATOR_PORT);
            Simulator sim = new Simulator(sensors);
            sourceServer.addWebService(SIMULATOR_IP, sim);
        	for( int iSensor = 0; iSensor < NUMBER_OF_SENSORS; iSensor++ )
        		createLocalMachineSensorSuite(sensors, "" + SENSOR_PORTS[iSensor]);
            sourceServer.start();
            sim.start();
    	}
    	else
    	{
        	//TODO Test this
        	addRealMuviumRPCServer(sensors);   	
    	}
    }
    
    /**
     * Get the current value of a sensor
     * @param sensor
     * @return
     */
    public int getSensorValue( int sensor ){
    	// TODO get the right sensor
    	Object sensorProxy = ((SensorServiceDescription)sensors.getSensors().get(sensor)).getProxy();
		if (sensorProxy instanceof IGenericSensorProxy) {
			return ((IGenericSensorProxy) sensorProxy).getSensorValue(0);
		}
		else if (sensorProxy instanceof IMotionDetectorProxy) {
			return ((IMotionDetectorProxy) sensorProxy).getMotionValue(0);
		}
		else if (sensorProxy instanceof IGasSensorUvmProxy) {
			return ((IGasSensorUvmProxy) sensorProxy).getSensorValue(0);
		}
		else {
	    	return 0;
		}
    }
    
    private static void addRealMuviumRPCServer( SensorMap sensorMap ){      
       RPCServer realUVMServer;
       for( int iPort = 0; iPort < NUMBER_OF_SENSORS; iPort++ )
       {
    	   Vector sensors = sensorMap.getGatewaySensors( SENSOR_PROXY_IP, "" + SENSOR_PORTS[iPort] );  
		   realUVMServer = new RPCServer( "" + SENSOR_PORTS[iPort] );
		   
		   for (Enumeration e = sensors.elements() ; e.hasMoreElements() ;) {
		       //You can setup multiple addresses, for example you can have a network of muvium devices
		       //programmed with different addresses listening to a multi-drop serial/radio/infrared network
		       //WebServices is request/respond so there are no collisions.
		       realUVMServer.addWebService( REAL_SENSOR_IP ); //Add a forwarding service for 10.1.1.10 
		       realUVMServer.start();

		       /* 
		       // USE THIS????
			   SensorServiceDescription sensor = (SensorServiceDescription) e.nextElement();       
			   realUVMServer.addWebService(sensor.getIpAddress(), 
					   sensor.createService(SENSOR_PROXY_IP,???,???)
			   ) ;
		        */

		   }
		   realUVMServer.start();
       }

   }
                   
   private static void createLocalMachineSensorSuite(SensorMap sensorMap
		   , String port )
   {     
       RPCServer uvmServer = new RPCServer(port, 1, 57600);

	   //RPCServer sensorSuitePort = new RPCServer(port);     
	   Vector sensors = sensorMap.getGatewaySensors( SENSOR_PROXY_IP, port);  
	   for (Enumeration e = sensors.elements() ; e.hasMoreElements() ;) {
		   SensorServiceDescription sensor = (SensorServiceDescription) e.nextElement();       
		   uvmServer.addWebService(sensor.getIpAddress(), 
				   sensor.createService(SENSOR_PROXY_IP,SIMULATOR_PORT,SIMULATOR_IP)
		   ) ;
	   }     
	   uvmServer.run();
   }
   
    public static void main(String[] args) {       
    	System.out.println("Starting GasSensor sample interface 2.0");
    	SensorMap sensors = new SensorMap();
    	
//    	runSim (sensors);
    	
        //Create the WebService enabled Simulator on 10.1.1.1 port Port80 gateway
        RPCServer simulatorServer = new RPCServer(SIMULATOR_PORT);
        Simulator sim = new Simulator(sensors);
        simulatorServer.addWebService(SIMULATOR_IP, sim);
        createLocalMachineSensorSuite(sensors, "83");
        createLocalMachineSensorSuite(sensors, "84");       
        createLocalMachineSensorSuite(sensors, "102");       
        simulatorServer.start();       
        sim.start();
    }
    

}
