package org.uva.gui;

/**
 * Interface to change the sensor values that are in a AgentProperty object
 */

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

import org.uva.object.*;

public class AgentControlGui extends JPanel implements ActionListener,
    ChangeListener
{
    private JTextField trailLength, agentNr;
    private JSlider persistence, sociability,
            property;
    private JTextField persistenceField, sociabilityField, propertyField;

    private AgentProperties properties;
    private ControlGui control;
    private JCheckBox ppBox = new JCheckBox("P/P" );

    public AgentControlGui(AgentProperties properties, ControlGui control)
    {
        this.control = control;
        this.properties = properties;
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        // add sleders, text fields etc. to the panel
        JPanel p3 = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        this.add(p3);
        p3.add(new JLabel("persistence"));
        p3.add(persistence = new JSlider(0, 100));
        p3.add(persistenceField = new JTextField(3));
        persistenceField.setEditable(false);

        JPanel p4 = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        this.add(p4);
        p4.add(new JLabel("sociability"));
        p4.add(sociability = new JSlider(0, 100));
        p4.add(sociabilityField = new JTextField(3));
        sociabilityField.setEditable(false);

        JPanel p5 = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        this.add(p5);
        p5.add(new JLabel("property"));
        p5.add(property = new JSlider(0, 100));
        p5.add(propertyField = new JTextField(3));
        propertyField.setEditable(false);

        JPanel p1 = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        this.add(p1);
        p1.add( ppBox );
        p1.add(new JLabel("NrOfAgents"));
        p1.add(agentNr = new JTextField(3));
        p1.add(new JLabel("TrailLength"));
        p1.add(trailLength = new JTextField(3));

        // add actionlisteners
        trailLength.addActionListener(this);
        persistence.addChangeListener(this);
        sociability.addChangeListener(this);
        property.addChangeListener(this);
        agentNr.addActionListener(this);
        ppBox.addActionListener( this );

        ppBox.setBackground(properties.color);
        this.setBackground(properties.color);
        p1.setOpaque(false);
        p3.setOpaque(false);
        p4.setOpaque(false);
        p5.setOpaque(false);

        startVariables();
        setVariables();
    }

    private void startVariables()
    {
        agentNr.setText("50");
        trailLength.setText("10");
        persistence.setValue(50);
        sociability.setValue(50);
        property.setValue(50);
    }

    // implement the changes
    public void actionPerformed(ActionEvent e)
    {
        if(e.getSource()==agentNr)
        {
            control.stopGeneration();
            try
            {
                Thread.sleep(100);
            }
            catch(Exception ee){}
            control.grid.initGrid(control.getGridWidth(), control.getGridHeight(), control.agentProperties );
            control.gridGui.setGrid(control.grid);
        }
        setVariables();
    }

    // implement changes
    public void stateChanged(ChangeEvent e)
    {
        if(e.getSource()==persistence)
        {
            setVariables();
        }
        else if(e.getSource()==sociability)
        {
            setVariables();
        }
        else if(e.getSource()==property)
        {
            setVariables();
        }
    }

    // set the new values in AgentProperties
    private void setVariables()
    {
        properties.persistence = persistence.getValue();
        properties.property = property.getValue();
        properties.sociability = sociability.getValue();
        properties.predatorPrey = ppBox.isSelected();

        try
        {
            properties.trailLength = Integer.parseInt(trailLength.getText());
        }
        catch(Exception e)
        {
            trailLength.setText("Err");
        }

        try
        {
            properties.agentCount = Integer.parseInt(agentNr.getText());
        }
        catch(Exception e)
        {
            agentNr.setText("Err");
        }

        persistenceField.setText(persistence.getValue()+"");
        sociabilityField.setText(sociability.getValue()+"");
        propertyField.setText(property.getValue()+"");
    }

}
