package org.uva.gui;

import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.uva.object.AgentProperties;

public class SensorConnectorGUI extends JFrame implements ActionListener{

	private static final double DONT_USE = Integer.MAX_VALUE;
	
	private JButton updateButton = new JButton( "Update" );
	
	public static final int NUMBER_OF_SENSORS = 16; 

	private GenerationHandler generationHandler = null;
	private AgentProperties properties = null;
	
	private JComboBox sensorList; 

	private SettingPanel sensorBox;
	private SettingPanel speedBox;
	private SettingPanel persistenceBox;
	private SettingPanel sociabilityBox;
	private SettingPanel propertyBox;
	private SettingPanel lengthBox;
	
	private SensorSetting[] settings = new SensorSetting[NUMBER_OF_SENSORS];
	
	
	public void setProperties(AgentProperties properties) {
		this.properties = properties;
	}

	public SensorConnectorGUI( )
	{
		super( "Sensor setup" );
		Container cp = this.getContentPane();
		cp.setLayout( new GridLayout(10,1) );

		String[] sensorNames = new String[NUMBER_OF_SENSORS];
		for( int i = 1; i <= NUMBER_OF_SENSORS; i++ )
		{
			sensorNames[i-1]="Sensor " + i;
			settings[i-1] = new SensorSetting();
		}

		sensorList = new JComboBox( sensorNames );

		cp.add( sensorList );
		
		cp.add( new JLabel( "Set the sensor input range") );		
		cp.add( sensorBox = new SettingPanel( "Input" )  );
		sensorBox.setDefaultTo( 1023 );
		sensorBox.setToValue( 1023 );

		cp.add( new JLabel( "Set the range of the properties") );
		cp.add( speedBox = new SettingPanel( "Speed" )  );
		cp.add( persistenceBox = new SettingPanel( "Persistence" )  );
		cp.add( sociabilityBox = new SettingPanel( "Sociability" )  );
		cp.add( propertyBox = new SettingPanel( "Property" )  );
		cp.add( lengthBox = new SettingPanel( "Trail length" )  );
		
		cp.add( updateButton );
		
		updateButton.addActionListener( this );
		sensorList.addActionListener( this );
		
		this.setBounds( 100, 400, 500, 400 );
		this.setVisible( true );
		this.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		
	}
	
	protected void updateEvent( SettingPanel panel, double fromValue, double toValue ){

		int index = sensorList.getSelectedIndex();
		if( index == -1 ){
			sensorList.setSelectedIndex( 0 );
			index = 0;
			return;
		}
		
		if( panel == sensorBox )
		{
			this.settings[index].sensorFrom = fromValue;
			this.settings[index].sensorTo = toValue;
		}
		if( panel == speedBox )
		{
			this.settings[index].speedFrom = fromValue;
			this.settings[index].speedTo = toValue;
		}
		
		if( panel == persistenceBox )
		{
			this.settings[index].persistenceFrom = fromValue;
			this.settings[index].persistenceTo = toValue;
		}
		if( panel == sociabilityBox )
		{
			this.settings[index].sociabilityFrom = fromValue;
			this.settings[index].sociabilityTo = toValue;
		}
		if( panel == propertyBox )
		{
			this.settings[index].propertyFrom = fromValue;
			this.settings[index].propertyTo = toValue;
		}
		if( panel == lengthBox )
		{
			this.settings[index].lengthFrom = fromValue;
			this.settings[index].lengthTo = toValue;			
		}

		
	}
	
	/**
	 * Set the new value for the given sensor
	 * @param newValue
	 * @param sensor 
	 */
	public void setSensorValue( int sensor, int newValue ){

		double value = this.settings[sensor].getSliderValue( 
				this.settings[sensor].speedFrom
				, this.settings[sensor].speedTo
				, (double)newValue );
		if( value != DONT_USE && this.generationHandler != null )
		{
			generationHandler.setTimer( (int) value );
		}
			
		if( this.properties == null )
			return;

		value = this.settings[sensor].getSliderValue( 
				this.settings[sensor].persistenceFrom
				, this.settings[sensor].persistenceTo
				, (double)newValue );
		if( value != DONT_USE  )
		{
			this.properties.persistence = (int)value;
		}

		value = this.settings[sensor].getSliderValue( 
				this.settings[sensor].sociabilityFrom
				, this.settings[sensor].sociabilityTo 
				, (double)newValue );
		if( value != DONT_USE  )
		{
			this.properties.sociability = (int)value;
		}

		value = this.settings[sensor].getSliderValue( 
				this.settings[sensor].propertyFrom 
				, this.settings[sensor].propertyTo  
				, (double)newValue );
		if( value != DONT_USE  )
		{
			this.properties.property = (int)value;
		}
		
		value = this.settings[sensor].getSliderValue( 
				this.settings[sensor].lengthFrom 
				, this.settings[sensor].lengthTo  
				, (double)newValue );
		if( value != DONT_USE  )
		{
			this.properties.trailLength = (int)value;
		}

				
		if( value != DONT_USE  )
			System.out.println( value );
	}


	
	private class SettingPanel extends JPanel implements ActionListener {
		JCheckBox checkBox;
		JTextField fromField = new JTextField( "0" );
		JTextField toField = new JTextField( "100" );
		
		private double defaultFrom = 0;
		private double defaultTo = 100;
		
		
		public SettingPanel( String text )
		{
			this.setLayout( new GridLayout( 1,4 ) );
			checkBox = new JCheckBox( text );
			add( checkBox );
			add( new JLabel( "Range:" ) );
			add( fromField );
			add( new JLabel( "-", JLabel.CENTER ) );
			add( toField );
			
			fromField.addActionListener( this );
			toField.addActionListener( this );
			checkBox.addActionListener( this );
			
		}

		/**
		 * Set the text value in the from field
		 * @param value
		 */
		public void setFromValue( double value ){
			if( value == DONT_USE ){
				checkBox.setSelected( false );
				value = defaultFrom;
			}
			fromField.setText( "" + (int)value );
		}
		
		public void select(){
			checkBox.setSelected( true );
		}

		/**
		 * Set the text value in the to field
		 * @param value
		 */
		public void setToValue( double value ){
			if( value == DONT_USE ){
				checkBox.setSelected( false );
				value = defaultTo;
			}
			toField.setText( "" + (int)value );
		}

		
		public void actionPerformed(ActionEvent e) {
			try{
				double from;
				double to;
				if( checkBox.isSelected() )
				{
					from = Double.parseDouble( fromField.getText() );				
					to = Double.parseDouble( toField.getText() );
				}
				else
				{
					from = to = DONT_USE;
				}
				updateEvent( this, from, to );
			}catch(Exception exc)
			{
				
			}
			
		}

		public void setDefaultFrom(double defaultFrom) {
			this.defaultFrom = defaultFrom;
		}

		public void setDefaultTo(double defaultTo) {
			this.defaultTo = defaultTo;
		}

	}
	
	
	
	
	private class SensorSetting
	{
		public double sensorFrom = DONT_USE;
		public double sensorTo = DONT_USE;

		public double speedFrom = DONT_USE;
		public double speedTo = DONT_USE;

		public double sociabilityFrom = DONT_USE;
		public double sociabilityTo = DONT_USE;

		public double persistenceFrom = DONT_USE;
		public double persistenceTo = DONT_USE;

		public double propertyFrom = DONT_USE;
		public double propertyTo = DONT_USE;

		public double lengthFrom = DONT_USE;
		public double lengthTo = DONT_USE;

		/**
		 * Convert the sensor value to a valid value for the slider of selected property 
		 * @param from
		 * @param to
		 * @param value
		 * @return
		 */
		public double getSliderValue( double from, double to, double value ){
			if( from == DONT_USE || to == DONT_USE
					|| sensorTo == DONT_USE || sensorFrom == DONT_USE )
				return DONT_USE;
			double sensorRange = sensorTo - sensorFrom;
			double valueRatio = Math.min( Math.max( value - sensorFrom, 0 ), sensorRange ) / sensorRange;

			double sliderRange = to - from;
			double slideValue = from + sliderRange * valueRatio;
			
			return slideValue;
		}

	}




	public void setGenerationHandler(GenerationHandler generationHandler) {
		this.generationHandler = generationHandler;
	}

	public void actionPerformed(ActionEvent e) {
		if( e.getSource() == updateButton )
		{
			sensorBox.actionPerformed( e );
			speedBox.actionPerformed( e );
			persistenceBox.actionPerformed( e );
			propertyBox.actionPerformed( e );
			sociabilityBox.actionPerformed( e );
			lengthBox.actionPerformed( e );
			/*setSensorValue( 0, 0 );
			setSensorValue( 0, 1023 );
			setSensorValue( 0, 512 );*/
			
			
		}
		else // sensor list
		{
			int index = sensorList.getSelectedIndex();
			if( index == -1 ) 
				return;
	
			sensorBox.select();
			sensorBox.setFromValue( this.settings[index].sensorFrom );
			sensorBox.setToValue( this.settings[index].sensorTo );
	
			speedBox.select();
			speedBox.setFromValue( this.settings[index].speedFrom );
			speedBox.setToValue( this.settings[index].speedTo );
	
			
			persistenceBox.select();
			persistenceBox.setFromValue( this.settings[index].persistenceFrom );
			persistenceBox.setToValue( this.settings[index].persistenceTo );
	
	
			sociabilityBox.select();
			sociabilityBox.setFromValue( this.settings[index].sociabilityFrom );
			sociabilityBox.setToValue( this.settings[index].sociabilityTo );
			
			propertyBox.select();
			propertyBox.setFromValue( this.settings[index].propertyFrom );
			propertyBox.setToValue( this.settings[index].propertyTo );

			lengthBox.select();
			lengthBox.setFromValue( this.settings[index].propertyFrom );
			lengthBox.setToValue( this.settings[index].propertyTo );

		}		
	}
		
}
