package org.uva.object;

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */

import java.awt.*;
import java.util.*;

public class Agent
{
    public Color color; // agent color
    public Point curPosition, nextPosition; // current and next position of the agent

    private int id; // agent id 
    protected int direction; //and direction
    private boolean persistenceFlip = false; // used in persistence

    public int group; // group id of the agent
    
    /**
     * The grid 
     */
    protected Grid agentGrid;
    
    protected Dimension gridSize;
    
    public Agent(int id, int group,Color color, Point curPosition, Grid grid )
    {
        this.id = id;
        this.color = color;
        this.curPosition = curPosition;

        this.group = group;

        this.direction = random();
        this.agentGrid = grid;
        GridElement[][] elements = grid.getGrid();
        this.gridSize = new Dimension( elements.length, elements[0].length );
    }

    /**
     * Change this method when you want to add/change behaviour of the agent
     * @param flat boolean
     * @param agents Hashtable agent objects
     * @param grid GridElement[][] in the grid we can get the agent id of a grid element
     * and by using this id, we can get the Agent object from the hashtable
     * @param p AgentProperties the behaviour values of the agent
     */
    public void nextPosition(boolean flat, Hashtable agents, GridElement[][] grid,
                             AgentProperties p)
    {
        int persistence = p.persistence;
        int sociability = p.sociability;
        int property = p.property;

        
        // determine the direction using pesistence
        int direction = persistence(persistence);

        //chance of the property sensor
        boolean c = chance(property);
        // if positive chance
        if(c||direction==-1)
        {
            // calculate sociability
            int soc = sociability(flat, agents, grid,
                                  sociability);
            // we can find a better direction because
            // we have found a neighbour agent
            if(soc!=-1)
            {
                direction = soc;
            }
        }

        // if both persistence and sociability fails, then do random walk
        // this probably never happens
        if(direction==-1)
        {
            direction = random();
        }

        // calculate the next position using the current position and
        // the calculated direction above
        Point p1 = neighbourPoint(direction, curPosition, flat, grid);

        // this is very important!!!!
        // the actual goal of this method is to set these two variables
        nextPosition = p1;
        this.direction = direction;
    }

    /**
     * Calculate quadratic distance
     * @param nextAgent
     * @return
     */
    public int distance( Agent nextAgent, boolean gridIsFlat )
    {
    	if( nextAgent.curPosition == null )
    		return Integer.MAX_VALUE;
    	int d;
    	if( gridIsFlat )
    	{
    		d = (int)( Math.pow( nextAgent.curPosition.x - curPosition.x, 2 )
    			+ Math.pow( nextAgent.curPosition.y - curPosition.y, 2 ) );
    	}
    	else
    	{
    		int dx = Math.abs(nextAgent.curPosition.x - curPosition.x);
    		int dy = Math.abs(nextAgent.curPosition.y - curPosition.y);
    		if( dx > gridSize.width / 2 )
    			dx = gridSize.width - dx;

    		if( dy > gridSize.height / 2 )
    			dy = gridSize.height - dy;

    		d = dx*dx + dy*dy;    		
    	}
    	
    	return d;
    }


    /**
     * Check if we look in the direction of the point (or the points < 4 away)
     * @param translation
     * @return
     */
    public boolean isPointVisible( Point translation )
    {
    	if( Math.abs( translation.x ) < 3 
    			&& Math.abs( translation.y ) < 3 )
    		return true;
    	
    	final int x = translation.x;
    	final int y = translation.y;
    	
        switch( this.direction )
        {
        case 1: // left down
        	return x <= 0 
        			&& y <= 0; 
            //return gridBounds(flat, grid, new Point(point.x - 1, point.y - 1));
        case 2: // down
        	return y >= 0
        			&& y <= -x
        			&& y <= x; 
            //return gridBounds(flat, grid, new Point(point.x, point.y - 1));
        case 3: // right down
        	return x >= 0 
        			&& y <= 0; 
            //return gridBounds(flat, grid, new Point(point.x + 1, point.y - 1));
        case 4: // left
        	return x <= 0
        			&& y >= x 
					&& y <= -x;
            //return gridBounds(flat, grid, new Point(point.x - 1, point.y));
        case 6: // right
        	return x >= 0 
					&& y >= -x
					&& y <= x; 
            //return gridBounds(flat, grid, new Point(point.x + 1, point.y));
        case 7: // left up 
        	return x <= 0 
        			&& y >= 0; 
            //return gridBounds(flat, grid, new Point(point.x - 1, point.y + 1));
        case 8:
        	return y >= 0 
					&& y >= -x
					&& y >= x; 
            //return gridBounds(flat, grid, new Point(point.x, point.y + 1));
        case 9:
        	return x >= 0 
        			&& y >= 0; 
            //return gridBounds(flat, grid, new Point(point.x + 1, point.y + 1));
        }
    	return false;
    }
    
    
    public Point getTranslation( Agent nextAgent, boolean gridIsFlat  )
    {
    	Point d;
    	if( nextAgent.curPosition == null )
    		return new Point( Integer.MAX_VALUE, Integer.MAX_VALUE );
    	
    	if( gridIsFlat )
    	{
    		d = new Point( nextAgent.curPosition.x - curPosition.x,
    						nextAgent.curPosition.y - curPosition.y );
    	}
    	else
    	{
    		int dx = nextAgent.curPosition.x - curPosition.x;
    		int dy = nextAgent.curPosition.y - curPosition.y;
    		if( dx > gridSize.width / 2 )
    			dx = dx - gridSize.width;
    		else if( dx < -gridSize.width / 2 )
    			dx = gridSize.width + dx;

    		if( dy > gridSize.height / 2 )
    			dy = dy - gridSize.height;
    		else if( dy < -gridSize.height / 2 )
    			dy = gridSize.height + dy;

    		d = new Point( dx,dy );    		
    	}
    	
    	return d;
    }
    
    private int sociability(boolean flat, Hashtable agents, GridElement[][] grid,
                             int sociability)
     {
       int x = findTrial(flat, agents, grid);
       if(x!=-1)
       {
           boolean chance = chance(sociability);
           // following another agent
           if (chance)
           {
               return x;
           }
           // opposite direction of another agent
           else
           {
               return 10-x;
           }
       }
       return -1;
   }

   // find the best trail in your neighbourhood
   private int findTrial(boolean flat, Hashtable agents, GridElement[][] grid)
   {
       int bestTrail = 10000;
       int bestDirection = -1;

       // for every point around you
       for(int x = 1; x<10; x++)
       {
           // except yourself
           if(x!=5)
           {
               Point p = neighbourPoint(x, curPosition, flat, grid);
               GridElement test = grid[p.x][p.y];
               // if you are not in your own trail and another agent of the same
               // agent type (group) as yourself
               if (test.getAgentId() != this.id&&test.group==this.group)
               {
                   // best trail, if there are more agents in
                   // your neighbourhood
                   if (bestTrail > test.getTrailCount())
                   {
                       bestTrail = test.getTrailCount();
                       Agent agent = (Agent) agents.get(new Integer(test.
                               getAgentId()));
                       // follow the other agent
                       if(agent!=null)
                           bestDirection = agent.direction;
                   }
               }
           }
       }
       return bestDirection;
   }

    private int persistence(int persistence)
    {
        int direction = -1;
        if(persistence==50)
        {
            direction = random();
        }
        else
        {
            boolean chance = chance(persistence);
            if(chance)
            {
                boolean c = chance(Math.max(persistence, 100-persistence));
                if(c==false)
                    return random();
                if(persistenceFlip!=chance)
                {
                    persistenceFlip = chance;
                    direction = 10 - this.direction;
                }
                else
                    direction = this.direction;
            }
            else
            {
                boolean c = chance(Math.max(persistence, 100-persistence));
                if(c==false)
                    return random();
                if(persistenceFlip!=chance)
                {
                    persistenceFlip = chance;
                    direction = 10 - this.direction;
                }
                else
                    direction = this.direction;

            }
        }
if(direction==-1)
            System.out.println("ERROR");
        return direction;
    }

    // does a random walk
    protected int random()
    {
        int direction = (int)(Math.random()*8);
        // direction in [ 0, 7 ]
        if( direction < 4 )
            direction++;
        else
            direction += 2;

        //int direction = 1 + (int)(Math.random()*9);
        // direction in [ 1, 9 ]

        return direction;
    }

    /**
     * Calculated the neighbour point given your current position and the direction you want to move to
     * @param nr int
     * @param point Point
     * @param flat boolean
     * @param grid GridElement[][]
     * @return Point
     */
    protected static Point neighbourPoint(int nr, Point point, boolean flat, GridElement[][] grid)
    {
        switch(nr)
        {
        case 1:
            return gridBounds(flat, grid, new Point(point.x - 1, point.y - 1));
        case 2:
            return gridBounds(flat, grid, new Point(point.x, point.y - 1));
        case 3:
            return gridBounds(flat, grid, new Point(point.x + 1, point.y - 1));
        case 4:
            return gridBounds(flat, grid, new Point(point.x - 1, point.y));
        case 5:
            return new Point(point);
        case 6:
            return gridBounds(flat, grid, new Point(point.x + 1, point.y));
        case 7:
            return gridBounds(flat, grid, new Point(point.x - 1, point.y + 1));
        case 8:
            return gridBounds(flat, grid, new Point(point.x, point.y + 1));
        case 9:
            return gridBounds(flat, grid, new Point(point.x + 1, point.y + 1));
        }
        System.out.println("ERROR "+nr);
        return new Point(point.x, point.y);
    }

    private static Point gridBounds(boolean flat, GridElement[][] grid, Point point)
    {
        if(point!=null)
            if(flat)
            {
                if(point.x<0)
                    point.setLocation(0, point.y);
                if(point.y<0)
                    point.setLocation(point.x, 0);
                if(point.x>=grid.length)
                    point.setLocation(grid.length-1, point.y);
                if(point.y>=grid[0].length)
                    point.setLocation(point.x, grid[0].length-1);
            }
            else
            {
                if(point.x<0)
                    point.setLocation(grid.length-1, point.y);
                if(point.y<0)
                    point.setLocation(point.x, grid[0].length-1);
                if(point.x>=grid.length)
                    point.setLocation(0, point.y);
                if(point.y>=grid[0].length)
                    point.setLocation(point.x, 0);
            }
        return point;
    }

    /**
     * Given a value between 0-100, it returns the chance
     * @param value int
     * @return boolean
     */
    protected boolean chance(int value)
    {
    	return (int)(Math.random()*101) <= value;
    }

    /**
     * Kill the agent and remove it from the grid
     *
     */
    public void die(){
    	nextPosition = null;
    }
    
    /**
     * Given a value between 0-1, it returns the chance
     * @param value double
     * @return boolean
     */
    protected boolean chance(double value)
    {
        
    	return(Math.random()<=value);
    }

    public void updatePosition()
    {
		curPosition = nextPosition;
        nextPosition = null;
    }
    
    public boolean isDead()
    {
    	return nextPosition == null;
    }
}
