package org.uva.object;

import java.awt.Color;

import java.awt.Point;
import java.util.Enumeration;
import java.util.Hashtable;

import learn.PredatorPreyAgentState;
import learn.QValueTable;

public class PredatorPreyAgent extends Agent {

	public static final int TYPE_PREDATOR = 0;
	public static final int TYPE_PREY 	= 1;
	
	protected static final double MAX_REWARD = 100; 
	protected static final double MAX_PENALTY = -100; 
	protected static final double PENALTY = -1; 
	protected static final double REWARD = 10;

	/**
	 * Decay rate
	 */
	private static final double DECAY = 0.75;

	/**
	 * Wanted area for a predator
	 */
	private static final double PREDATOR_AREA = 256.0;

	/**
	 * Wanted area for a prey
	 */
	private static final double PREY_AREA = 25.0;

	private double propertyValue = 0;
	
	/**
	 * simple flag to remove the agent from the tables
	 */
	protected boolean dead = false;
	
	/**
	 * Store the previous action
	 */
	protected int lastAction = 0;
	
	/**
	 * Viewing distance, quadratic ( distance^2 should be <= viewDistance 
	 */
	protected int viewDistance = 36; 
	
	/**
	 * Viewing distance for a prey
	 */
	private static final int PREY_VIEW_DISTANCE = 16;

	/**
	 * Viewing distance for a prey
	 */
	private static final int PREDATOR_VIEW_DISTANCE = 16;

	private int agentType = 0;
	
	private PredatorPreyAgentState currentState = null;
	private PredatorPreyAgentState previousState = null;;
	
	//private boolean isDead = false;
//	private int deadTimeOut = 0; 
//	private Point deadPosition = null;
	
	protected static QValueTable preyTable = new QValueTable();
	protected static QValueTable predatorTable = new QValueTable();

	private QValueTable qTable;
	
	/**
	 * Nr of predators
	 */
	private static int nPreys = 0;
	/**
	 * Nr of preys
	 */
	private static int nPredators = 0;
	
	
	/**
	 * Construct new predator or prey depending on the type
	 * @param type TYPE_PREDATOR or TYPE_PREY
	 */
	public PredatorPreyAgent( int id, int group, Color color, Point position, Grid grid, int type )
	{
		super( id, group, color, position, grid );
		this.agentType = type;
		//System.out.println( "Created: " + this.agentType );
		if( this.agentType == TYPE_PREDATOR )
		{
			nPredators++;
			this.qTable = predatorTable;
			this.viewDistance = PREDATOR_VIEW_DISTANCE;
		}
		else
		{
			nPreys++;
			this.qTable = preyTable;
			this.viewDistance = PREY_VIEW_DISTANCE;
		}
		
	}
	

	public int getViewDistance() {
		return viewDistance;
	}

	public void updatePosition()
	{
		super.updatePosition();
		previousState = currentState; 
		lastAction = direction;
	}

	public void setViewDistance(int viewDistance) {
		this.viewDistance = viewDistance;
	}

	
	
	/**
	 * Determine next position. Default for prey, smart for predator
	 */
	public void nextPosition( boolean flat, Hashtable agents, GridElement[][] grid, AgentProperties properties )
	{
		this.propertyValue = properties.property;
		/*if( this.dead )
		{
			this.deadTimeOut--;
			nextPosition = null;
			curPosition = null;
			if( this.deadTimeOut == 0 )
			{
				this.dead = false;
				this.curPosition = deadPosition;
				System.out.println( "Reviving @: " + this.curPosition );
			}
			else
			{
				return;
			}
			
		}*/
		if( this.dead )
			return;
		// don't know, sometimes happens
		if( this.curPosition == null )
			this.curPosition = new Point(0,0);

		currentState = new PredatorPreyAgentState( this, agents, flat );
		Hashtable localAgents = currentState.getLocalAgents();
		Hashtable caughtPreys = currentState.getCaughtPreys();
		double reward = determineReward( localAgents, caughtPreys, flat );

		
		if( this.agentType == TYPE_PREDATOR )
		{
			direction = this.getBestAction( properties, reward );
			nextPosition = neighbourPoint( direction, curPosition, flat, grid);
		}
		else
		{
			super.nextPosition( flat, agents, grid, properties );
		}
		// kill if there are too many agents
		checkOverPopulated();
		
		/*if( this.dead )
		{
			if( previousState != null )
			System.out.println( " Value: " + qTable.getQ( previousState, lastAction ) 
					+ " Better: " + qTable.getMaxQ( previousState) );
			deadPosition = nextPosition;
			nextPosition = null;
		}*/
	}
	
	private int getBestAction( AgentProperties properties, double reward )
	{

		int action;
		if( chance( properties.persistence ) && previousState != null )
		{
			action = lastAction;
		}
		else if( chance( properties.sociability ) && previousState != null )
		{
			action = random();
		} 
		else
		{
			action = qTable.getMaxAction( currentState );
			if( action == QValueTable.ACTION_RANDOM 
					|| action == QValueTable.ACTION_NOT_FOUND )
			{
				action = random();
			}
			/*else{
				if( previousState!= null )
					System.out.println( "Action " + action 
							+ " Value: " + qTable.getQ( previousState, action ));
			}*/
		}
			
		if( previousState != null )
			qTable.update( previousState, lastAction, reward, currentState );

		return action;
	}
	
	public double determineReward( Hashtable agents, Hashtable caughtPreys, boolean flat )
	{
		double reward = 0;

        // update agents
        Enumeration en = agents.keys();

		switch( this.agentType )
		{
		case TYPE_PREDATOR:
			
	        while(en.hasMoreElements() && !dead)
	        {
	        	Point location = (Point) en.nextElement();
	            //Integer distance = (Integer)en.nextElement();
	        	int distance = location.x*location.x+location.y*location.y;
	        	
	            Agent agent = (Agent)agents.get(location);
	            if( agent == this || agent.isDead() )
	            	continue;
	            
	            if( agent instanceof PredatorPreyAgent )
	            {
	            	switch( ((PredatorPreyAgent)agent).agentType )
	            	{
	            		case TYPE_PREY:
		            	//reward += MAX_REWARD * Math.pow( 0.2, distance.doubleValue() );
//	            		nAgents++;
			            	if( distance < 1 )
			            	{
			            		/*if( previousState != null )
			            			System.out.println( "Caught prey: " + qTable.getQ( previousState, lastAction ) );*/
			            		reward += MAX_REWARD;
			            		if( chance( getSpawnChance() ) )
			            		{
			            			//System.out.println( "New predator" );
			            			spawn();
			            		}
			            	}
			            	break;
	            		case TYPE_PREDATOR:
			            	if( distance < 1 )
			            	{
			            		//System.out.print( "Collission" );
			            		reward += MAX_PENALTY;
			            		//this.deadTimeOut = 15;
			            		//this.deadPosition = curPosition;
			            		// one out of 2 should die
			            		//if( chance( getDieChance() ) )
			            		//{
		            			die();
		            			if( nPredators < 1 )
		            				spawn();
			            		//}
			            	}
			            	break;
		            } // switch type
	            }
	             
	        } // end while
	        
            en = caughtPreys.keys();
	        while(en.hasMoreElements() && !dead)
	        {
	        	Point location = (Point) en.nextElement();
	            //Integer distance = (Integer)en.nextElement();
	        	double distance = Math.sqrt( location.x*location.x+location.y*location.y );
	        	reward = MAX_REWARD / distance;
	        }

	        
			if( reward == 0 )
			{
				reward = PENALTY;
//				nAgents = 1;
			}
			break;
		case TYPE_PREY: 
	        while(en.hasMoreElements() && !dead)
	        {
	        	Point location = (Point) en.nextElement();
	            //Integer distance = (Integer)en.nextElement();
	        	int distance = location.x*location.x+location.y*location.y;
	        	
	            Agent agent = (Agent)agents.get(location);
	            if( agent == this )
	            	continue;
	            
	            if( agent instanceof PredatorPreyAgent )
	            {
	            	switch( ((PredatorPreyAgent)agent).agentType )
	            	{
	            		case TYPE_PREDATOR:
			            	if( distance < 1 )
			            	{
			            		//System.out.print( "Dead" );
			            		reward += MAX_PENALTY;
			            		//this.deadTimeOut = 15;
			            		//this.deadPosition = curPosition;
			            		//if( chance(50) )
			            		//{
		            			die();
		            			if( nPreys <= 2 )
		            				spawn();
			            		//}
			            	}
			            	break;
	            		case TYPE_PREY:
			            	if( distance < 1 )
			            	{
			            		reward += MAX_REWARD;
			            		//System.out.println( "Spwan" );
			            		if( chance( getSpawnChance() ) )
			            			spawn();
			            	}
			            	break;
	            	} // switch type
	            }
	        }
			/*if( reward == 0 )
			{
				reward = REWARD;
			}*/
		} // end switch
		
		return reward;
	}
	
	/**
	 * Kill the agent
	 *
	 */
	public void die()
	{
		if( !dead )
		{
			switch( this.agentType )
			{
				case TYPE_PREDATOR:
					nPredators--;
					break;
				case TYPE_PREY:
					nPreys--;
					break;
			}
		}
		dead=true;

	}
	
	/**
	 * Add a new agent randomly on the grid
	 *
	 */
	public void spawn(){
		int area = this.gridSize.width * this.gridSize.height;
		Color color = this.color;
		switch( agentType )
		{
			case TYPE_PREDATOR:
				if( nPredators > 0 && area / nPredators < PREDATOR_AREA )
					return;
				break;
			case TYPE_PREY:
				if( nPreys > 1 && area / nPreys < PREY_AREA )
					return;
				color = new Color( (int)(Math.random() * 75 )
						, (int)(Math.random() * 256 )
						, (int)(Math.random() * 256 ));
				break;		
		}
		Point newPoint = this.agentGrid.getRandomPosition();
		this.agentGrid.addPPAgent( newPoint.x, newPoint.y, color, this.group, this.agentType );
	}
	/**
	 * Check if we should remove the agent from the grid
	 */
    public boolean isDead()
    {
    	return dead;
    }
    
    /**
     * Kill the agent if we are overpopulated
     */
    public void checkOverPopulated()
    {
    	if( this.isDead() )
    		return;
    	
		if( chance( this.getDieChance() ) )
			die();
    }

    /**
     * Chance of dieing, correct for overpopulation
     * @return
     */
    private double getDieChance()
    {
    	int area = this.gridSize.height*this.gridSize.width; 
    	if( this.isDead() )
    		return 1;
    	double dieChance = 1;
    	double ratio = 0;
    	switch( this.agentType )
    	{
    	case TYPE_PREY:
    		//ratio = (nPreys-2)*(nPreys-2) * 49.0 / area;
    		// wanted = area / ( 7 * 7 )
    		ratio = (nPreys-2) * PREY_AREA / area;
    		double factor = 0.001 + ( 0.009 * (100.0-this.propertyValue) / 100 );

    		//dieChance = 0.05 - 0.05 * Math.exp( - ratio );	
    		//dieChance = 0.01 - 0.01 * Math.pow( DECAY, ratio );
    		dieChance = factor - factor * Math.pow( DECAY, ratio );
    		
    		break;
    	case TYPE_PREDATOR:
    		ratio = (nPredators-1)*PREDATOR_AREA / area;
    		//dieChance = 0.10 - 0.10 * Math.exp( - ratio );	
    		dieChance = 0.005 - 0.005 * Math.pow( DECAY, ratio );
    		break;
    	}
    	return dieChance;
    }
    
    /**
     * Chance of spawning, correct for overpopulation
     * @return
     */
    private double getSpawnChance()
    {
    	int area = this.gridSize.height*this.gridSize.width; 
    	if( this.isDead() )
    		return 0;
    	double spawnChance = 1;
    	double ratio = 0;
    	switch( this.agentType )
    	{
    	case TYPE_PREY:
    		ratio = (nPreys-2) * PREY_AREA / area;
    		ratio = ratio * ratio;
    		//spawnChance = ( 0.01 + ( 0.09 * this.propertyValue / 100 ) ) * Math.exp( - ratio );
    		//spawnChance = 0.05 * Math.exp( - ratio );	
    		spawnChance = ( /*0.05 +*/ ( 0.2 * this.propertyValue / 100 ) ) * Math.pow( DECAY , ratio );
    		break;
    	case TYPE_PREDATOR:
    		ratio = (nPredators-1)*PREDATOR_AREA / area;
    		ratio = ratio * ratio;
    		//spawnChance = 0.15 * Math.exp( - ratio );	
    		spawnChance = 0.2 * Math.pow( DECAY, ratio );
    		break;
    	}
    	//System.out.println( spawnChance );
    	return spawnChance;
    }
    
    /**
     * Check if the predators or prey are equal
     * @return equal if same group and type
     */
    public boolean equals( Object o ){
    	if( o instanceof PredatorPreyAgent )
    	{
    		PredatorPreyAgent a = (PredatorPreyAgent)o;
    		return a.group == this.group 
    			&& a.agentType == this.agentType;
    	}
    	return false;
    }
    

    /**
     * Check if the predators or prey are equal
     * @return equal if same group and type
     */
    public int hashCode(  ){
    	return this.group * (this.agentType * 2 - 1 );
    }

    public int getAgentType(){
    	return this.agentType;
    }
}
