package org.uva.sensorServer;

import java.util.Enumeration;
import ws.interfaces.*;
import ws.proxyImpls.*;

/**
 * The similator is used to both represent and retrive data for certain scenarios
 * 
 * <p>
 * <b>NOTES:</b><br>
 * <ul>
 * <li>Version Mar 9, 2005 ( Created ) </li>
 * </ul>
 * 
 * @author James Caska
 * @see 
 */

//TODO: Field size must be collected by server and set to check boundaries.

public class SensorDataGetter extends Thread implements ISensorSim {
	private	SensorMap mySensorMap;
	
	public SensorDataGetter(SensorMap sensorMap){
		mySensorMap = sensorMap;
	}
	
	/**
	 * Access the simulated Sensor data
	 */
	public int getSensorValue(int field){
		double val = 1023.0 * Math.random();	
		if( field >=0 && field < 256){
			return (int) val;
		}else{
			return -1;
		}
	}
	
	/**
	 * Access the Sensor data through the interface
	 */
	
	public int getSensorData(int sensorId, int field) {
		SensorDescription sensor = mySensorMap.getSensor(sensorId);
		int value = -1;
		if (sensor.active) {
			Object sensorProxy = sensor.getProxy();		
			if (sensorProxy instanceof ISensorSimProxy) {
				value = ((ISensorSimProxy) sensorProxy).getSensorValue(field);
				if (field == 0) {
					System.out.println( "Getting Simulated Sensor:" +
						sensor.myDeviceIp + ":" + sensor.myGatewayPort+ " (" + sensor.myId + "-" + field + ")=" + value);
				}}
			else if (sensorProxy instanceof ISensorUvmProxy) {
				value = ((ISensorUvmProxy) sensorProxy).getSensorValue(field);
				if (field == 0) {
//					System.out.println( "Getting Uvm Sensor:" +
//							sensor.myDeviceIp + ":" + sensor.myGatewayPort+ " (" + sensor.myId + "-" + field + ")=" + value);
				}}
			else {
				System.out.println("Unknown Sensor:" + sensor.myDeviceIp
						+ ":" + sensor.myGatewayPort + " (" + sensor.myId + ")");
			}
		}
		else
			System.out.println("Sensor " + sensorId + " not activated");
		return value;
	}
	
	public String getName (String s1) {
		return "aSensor";
	}
	
	public void scanSensors() {
		for (Enumeration e = mySensorMap.getSensors().elements(); e
		.hasMoreElements();) {
			SensorDescription sensor = (SensorDescription) e.nextElement();	
			getSensorData(sensor.myId, 0);
		}
	}
	
	public void run(){   
		while( true ){        
			try{
				Thread.sleep(1000);
			}catch(Exception e){}
			scanSensors();	
		}
	}
}
