package org.uva.sensorServer;
/*
 * @(#)SensorServiceDescription.java Created on Mar 9, 2005
 *
 * Copyright 2003 to 2005 James Caska, muvium.com . All Rights Reserved
 */

import ws.impls.*;
import ws.proxyImpls.*;

/**
 * <p>
 * <b>NOTES:</b><br>
 * <ul>
 * <li>Version Mar 9, 2005 ( Created ) </li>
 * </ul>
 * 
 * @author James Caska
 * @see 
 */
public class SensorDescription{
  
    private static final int INTERFACE_UNKNOWN	 		= 0;
    private static final int INTERFACE_SIM_SENSOR 		= 1;
    private static final int INTERFACE_UVM_SENSOR 		= 2;
   
    int		myId;
    String 	myGatewayIp;
	String 	myGatewayPort;
	String 	myDeviceIp;
	int 	myServiceInterface;
	String 	myServiceType;
	Object	myProxy;
	Object 	myService;
	boolean active;
	
	SensorDescription(int id,
	        		String gatewayIp, 
					String gatewayPort,
					String deviceIp,
					String serviceInterface,
					String serviceType){
	    
	    myId				= 	id;
        myGatewayIp 		= 	gatewayIp;
		myGatewayPort 		= 	gatewayPort;
		myDeviceIp			= 	deviceIp;
		myServiceInterface	=	getInterfaceType( serviceInterface );
		myServiceType		=	serviceType;
		myProxy				= 	createProxy();
		active 				= 	false;
   }
	
	public String getIpAddress(){
	    return myDeviceIp;
	}
	
	private String createProxyFormatedDeviceIpAddress(){
	    return  myGatewayIp  + ":" +  myGatewayPort + "/http://"  + myDeviceIp ;
	}
	
	private int getInterfaceType( String interfacName ){   
	    if( interfacName.equals( "ISimSensor" )){
	        return INTERFACE_SIM_SENSOR;
	    }    
	    if( interfacName.equals( "IUvmSensor" )){
	        return INTERFACE_UVM_SENSOR;
	    }    
	    return INTERFACE_UNKNOWN;	    
	}
	
	public boolean matchGateway( String gatewayIp, String gatewayPort){
	    return 	gatewayIp.equals( myGatewayIp ) &&
	    		gatewayPort.equals( myGatewayPort );    
	}
	
	public void setActive(boolean value) {
		active = value;
	}
	
	public boolean getActive() {
		return active;
	}
	
	public String key(){
	    return "S" + myId;
	}
	
	public int getId(){
	    return myId;
	}
	
	public Object getProxy(){
	    return myProxy;
	}
	
	private Object createProxy(){
	    switch( myServiceInterface ){
	    	case INTERFACE_SIM_SENSOR:
	    	    return new ISensorSimProxy(createProxyFormatedDeviceIpAddress()); 
	   		case INTERFACE_UVM_SENSOR:
	    	    return new ISensorUvmProxy(createProxyFormatedDeviceIpAddress()); 
	    }
	    return null;
	}
	
	public Object createService( 	String simulatorProxy, 
	        						String simulatorPort,
	        						String simulatorIp ){
	        switch(myServiceInterface){
	        	case INTERFACE_SIM_SENSOR:
			        return new SensorSimImpl( myId,
	    					simulatorProxy, 
	    					simulatorPort,
	    					simulatorIp);
		   		case INTERFACE_UVM_SENSOR:
				    return new SensorUvmImpl( myId,
							simulatorProxy, 
							simulatorPort,
							simulatorIp);     
	        }
	    return null;
	}
}