package org.uva.sensorServer;

import java.util.Enumeration;
import java.util.Vector;
import com.muvium.web.services.gateway.RPCServer;

/**
 * Version 	0.0 28-01-2006 Created from reference app
 * This simulator and sensor interface allows the main program to
 * interface directly with a muVium device and read sensors, connected to
 * a microcontroller.
 * All communication is done through a WebService interface, that simplifies
 * the communication to a Remote Procedure Call.
 * The simulator is included to allow generating random values
 * that may be read as if a real device is attached, allowing tests.
 * 
 * @authors James Caska and Peter van Lith
 */

public class SensorServer {
	public static final String	SIMULATOR_PROXY_IP	 	= "127.0.0.1";   // Proxy
	public static final String 	SIMULATOR_PORT 			= "81";
	public static final String 	SIMULATOR_IP 			= "10.1.1.1";
	public static final String 	SIMULATOR_INTPORT 		= "90";			// Simulator interface
	public static final String 	SIMULATOR_DEV_IP 		= "10.1.1.1";
	public boolean sim;			// This variable controls real or simulated sensors
	SensorDataGetter simData;
	
	public SensorServer(boolean simulated) {       
		System.out.println("Starting sensor server 2.0");
		sim = simulated;
		SensorMap sensors = new SensorMap();
		if (!sim) {
			addRealMuviumRPCServer(sensors);   
		} else {			
			//Create the WebService enabled Simulator on 10.1.1.1 port Port80 gateway
			RPCServer simulatorServer = new RPCServer(SIMULATOR_PORT);       
			simData = new SensorDataGetter(sensors);
			simulatorServer.addWebService(SIMULATOR_IP, simData);
			createSimulatedSensor(sensors, "83");  // Add two simulated sensors
			createSimulatedSensor(sensors, "84");       
			simulatorServer.run();       
		}
	}
	
	private static void addRealMuviumRPCServer(SensorMap sensorMap){      
		RPCServer realUVMServer = new RPCServer("101", 1, 57600);
		//You can setup multiple addresses, for example you can have a network of muvium devices
		//programmed with different addresses listening to a multi-drop serial/radio/infrared network
		//WebServices is request/respond so there are no collisions.
		sensorMap.activateSensors("127.0.0.1", "101");
		realUVMServer.addWebService( "10.1.1.10" ); //Add a forwarding service for 10.1.1.10 
		realUVMServer.start();              
	}
	
	private static void createSimulatedSensor(SensorMap sensorMap, String port){     
		RPCServer sensorPort = new RPCServer(port);  
		sensorMap.activateSensors("127.0.0.1", port);  
		Vector sensors = sensorMap.getGatewaySensors("127.0.0.1", port);  
		for (Enumeration e = sensors.elements() ; e.hasMoreElements() ;) {
			SensorDescription sensor = (SensorDescription) e.nextElement(); 
			sensorPort.addWebService(sensor.getIpAddress(), 
					sensor.createService(SIMULATOR_PROXY_IP,SIMULATOR_PORT,SIMULATOR_IP)
			) ;
		}     
		sensorPort.run();
	}
	
	public int getSensorData(int field) {
		if (sim)
			return simData.getSensorData(1, field);  // Get simulated sensor 1
		else
			return simData.getSensorData(3, field);  // Get real sensor on 3
	}
	
}
