package ws.test;

//import SensorMap;
import java.util.Enumeration;
import java.util.Vector;
import com.muvium.web.services.gateway.RPCServer;

/**
 * Version 	0.0 09-03-2005 Created
 * 			0.1 02-04-2005 Added string interfaces
 * 			0.2 09-04-2005 Added Simulator interface
 * 			1.4 22-04-2005 Included DPN output sensor
 * 
 * @authors James Caska and Peter van Lith
 */

public class GasSensorProject {

    public static final String	SIMULATOR_PROXY_IP	 	= "127.0.0.1";   // Proxy
    public static final String 	SIMULATOR_PORT 			= "81";
    public static final String 	SIMULATOR_IP 			= "10.1.1.1";
    public static final String 	SIMULATOR_INTPORT 		= "90";			// Simulator interface
    public static final String 	SIMULATOR_DEV_IP 		= "10.1.1.1";

    private static void addRealMuviumRPCServer( SensorMap sensors ){      
       RPCServer realUVMServer = new RPCServer("101", 1, 57600);
       //You can setup multiple addresses, for example you can have a network of muvium devices
       //programmed with different addresses listening to a multi-drop serial/radio/infrared network
       //WebServices is request/respond so there are no collisions.
        realUVMServer.addWebService( "10.1.1.10" ); //Add a forwarding service for 10.1.1.10 
        realUVMServer.start();              
   }
                   
   private static void createLocalMachineSensorSuite(SensorMap sensorMap, 
		   String port){     
	   RPCServer sensorSuitePort = new RPCServer(port);     
	   Vector sensors = sensorMap.getGatewaySensors( SIMULATOR_PROXY_IP, port );  
	   for (Enumeration e = sensors.elements() ; e.hasMoreElements() ;) {
		   SensorServiceDescription sensor = (SensorServiceDescription) e.nextElement();
		   // requests to the sensor are routed to the proxy @ SIMULATOR_PROXY_IP : SIMULATOR_PORT ?? 
		   sensorSuitePort.addWebService(sensor.getIpAddress(),  
				   sensor.createService(SIMULATOR_PROXY_IP,SIMULATOR_PORT,SIMULATOR_IP)
		   ) ;
	   }     
	   sensorSuitePort.run();
   }
   
    public static void main(String[] args) {   
    	/**
    	 * 
    	 * A server for the simulator is created, 
    	 * working on  SIMULATOR_IP : SIMULATOR_PORT
    	 * 
    	 * Simulator for the sensor values is created 
    	 * and added to the simulator server
    	 * 
    	 * Sensors are created
    	 *  
    	 * Sensors are connected
    	 */
    	
    	
    	System.out.println("Starting GasSensor sample interface 2.0");

//    	addRealMuviumRPCServer(sensors);   	
//    	runSim (sensors);
    	
        //Create the WebService enabled Simulator on 10.1.1.1 port Port80 gateway
        RPCServer simulatorServer = new RPCServer(SIMULATOR_PORT);
        Simulator sensorValueGenerator = new Simulator( new SensorMap() );
        // connect the sensor value generator to the proxy
        simulatorServer.addWebService(SIMULATOR_IP, sensorValueGenerator );

        // generate sensors
    	SensorMap sensors = new SensorMap();
    	sensors.addSensor(new SensorServiceDescription(1, SIMULATOR_PROXY_IP,"83",	"10.1.1.7",	"IMotionSensor","sim" ));
    	sensors.addSensor(new SensorServiceDescription(2, SIMULATOR_PROXY_IP,"83",	"10.1.1.8",	"IGasSensor","sim" ));
    	sensors.addSensor(new SensorServiceDescription(3, SIMULATOR_PROXY_IP,"84", "10.1.1.8", "IGasSensor","sim"));
    	sensors.addSensor(new SensorServiceDescription(4, SIMULATOR_PROXY_IP,"102", "10.1.1.10", "IUvmSensor","uvm"));

        //connect sensors
        Simulator sim = new Simulator(sensors);
        createLocalMachineSensorSuite(sensors, "83");
        createLocalMachineSensorSuite(sensors, "84");       
        createLocalMachineSensorSuite(sensors, "102");       
        simulatorServer.start();       
        sim.start();
    }
    
}
