package ws.test;
/**
 * Sensor.java 
 * Sensor Network Simulator Project
 *  
 * This class represents the sensors in the World. A sensor has an ID, 
 * location, orientation, icon for the GUI, status, level of alertness and 
 * either subsensors or measures or both.
 * 
 * Authors: Maarten Corzilius, Michiel Kamermans, Tijn Schmits, Mark Smids
 * Second authors: David de Bos, Sanne Korzec, Timothy Lamballais Tessensohn
 * Last Modification: 2005-06-29 
 * 
 * 29-06-2005 sensorProxy for Real sensors added
 */

// TODO: Must generate default values for WS interface definitions

import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.JFrame;
import ws.impls.GenericSensorSimImpl;
import ws.impls.MotionSensorSimImpl;
import ws.impls.SimulatorSimImpl;
import ws.proxyImpls.*;
import com.muvium.web.services.gateway.RPCServer;

public class Sensor {
	private int sensorID;
	public String type;
	private String name;
	// WS definitions
	public String gatewayIp;			// Sensor Gateway
	public String gatewayPort;			// Port
	public String deviceIp;				// IP address
	public String interfaceType;	    // Interface Type
	public String serviceType;			// Service type indication
	public Object sensorProxy;
	public Object service;				// Service Object
	
    public static final String	SIMULATOR_PROXY_IP	 	= "127.0.0.1";
    public static final String 	SIMULATOR_PORT 			= "81";
    public static final String 	SIMULATOR_IP 			= "10.1.1.1";
    public static final String 	SIMULATOR_INTPORT 		= "90";
    public static final String 	SIMULATOR_DEV_IP 		= "10.1.1.1";
    
    //  XML looks like this:
    //	sensorID="0" gatewayIP="127.0.0.1" devicePort="82" deviceIP="10.1.1.7" interfaceType="IGenericSensor" serviceType="sim"/>

	// Constructors
	// Default constructor
	public Sensor() {
		super();
		this.sensorID=0;
		this.type = "Sensor";
		this.gatewayIp = "";
		this.deviceIp = "";
		this.gatewayPort = "";
		this.serviceType = "sim";
		this.sensorProxy = null;
		this.service = null;
	}

	// Constructor with ID string
	public Sensor(int ID)
	{
		this.sensorID=ID;
		this.type = "Sensor";
		this.gatewayIp = "";
		this.deviceIp = "";
		this.gatewayPort = "";
		this.serviceType = "sim";
		this.sensorProxy = null;
	}

	public Sensor (int id,
	        String gatewayIp, 
			String gatewayPort,
			String deviceIp,
			String type,
			String serviceType){	    
		this.sensorID = id;
		this.type = "Sensor";
		this.gatewayIp = gatewayIp;
		this.deviceIp = deviceIp;
		this.gatewayPort = gatewayPort;
		this.interfaceType = type;
		this.serviceType = serviceType;
//		this.sensorProxy = createSensorProxy(deviceIp);
//		this.service = createService(sensorProxy,gatewayPort,gatewayIp);
   }
	// Getter methods
	public String getSensorID() {return "S" + this.sensorID;}
	public String getName() {return this.name;}
	public String getType() {return this.type;}
	public String getGatewayIP() {return this.gatewayIp;}
	public String getDeviceIP() {return this.deviceIp;}
	public String getDevicePort() {return this.gatewayPort;}
	public String getServiceType() {return this.serviceType;}

	// <Sensor id="P.vLith" name="Phone P van Lith" type="SmartPhone" location="N 06 30 15 E 80 01 15" orientation="-1" 
	// sensorID="0" gatewayIP="127.0.0.1" gatewayPort="82" deviceIP="10.1.1.7" serviceType="sim"/>

	public void setGatewayIP(String number, String timestamp) {
		this.gatewayIp = number;
	}

	public void setDevicePort(String number, String timestamp) {
		this.gatewayPort = number;
	}

	public void setDeviceIP(String number, String timestamp) {
		this.deviceIp = number;
	}

	public void setServiceType(String number, String timestamp) {
		this.serviceType = number;
	}
	
	public static void createSensorInterface(RPCServer aPortServer) {
		Object aService = null;
		aService = new SimulatorSimImpl(SIMULATOR_PROXY_IP, SIMULATOR_PORT, SIMULATOR_IP);
		if (aService != null)
		{
			aPortServer.addWebService(SIMULATOR_DEV_IP, aService);
			System.out.println("Created WS for simulator on: " + SIMULATOR_DEV_IP);
		}
	}
	    
	public void createLocalSensor(RPCServer aPortServer) {
		Object aService = null;
		aService = this.createService(SIMULATOR_PROXY_IP, SIMULATOR_PORT, SIMULATOR_IP);
		if (aService != null)
		{
            if(getServiceType().equals("uvm")){
                aPortServer.addWebService(this.getDeviceIP() );
            }else{
                aPortServer.addWebService(this.getDeviceIP(), aService);
            }
			this.service = aService;
			System.out.println("Created WS for: " + this.getDeviceIP());
		}
	}
	
	public void createSensorProxy(String ip) {
		this.sensorProxy = new IGenericSensorProxy(ip);
	}

	public void createSensorUvmProxy(String ip) {
		this.sensorProxy = new IGasSensorUvmProxy(ip);
	}

	public Object createService(String simulatorProxy, String simulatorPort,
			String simulatorIp) {

		if (this.serviceType.equals("uvm")) {
			System.out.println("Created local UVM Gas Sensor: " + this.sensorID);
			return new GenericSensorSimImpl(sensorID, simulatorProxy,
					simulatorPort, simulatorIp);
		}
		else {
			if (this.type.equalsIgnoreCase("Gas")) {
				System.out.println("Created local Gas Sensor: " + this.sensorID);
				return new GenericSensorSimImpl(sensorID, simulatorProxy,
						simulatorPort, simulatorIp);
			}
			else {
				System.out.println("Unknown sensor:" + this.type + " " + this.sensorID);
				return null;
			}
		}
	}
}