package ws.test;
/*
 * @(#)SensorServiceDescription.java Created on Mar 9, 2005
 *
 * Copyright 2003 to 2005 James Caska, muvium.com . All Rights Reserved
 */

import ws.impls.*;
import ws.proxyImpls.*;

/**
 * <p>
 * <b>NOTES:</b><br>
 * <ul>
 * <li>Version Mar 9, 2005 ( Created ) </li>
 * </ul>
 * 
 * @author James Caska
 * @see 
 */
public class SensorServiceDescription{
  
    private static final int INTERFACE_UNKNOWN	 		= 0;
    private static final int INTERFACE_GAS_SENSOR 		= 1;
    private static final int INTERFACE_MOTION_SENSOR 	= 2;
    private static final int INTERFACE_UVM_SENSOR 		= 3;
   
    int		myId;
    String 	myGatewayIp;
	String 	myGatewayPort;
	String 	myDeviceIp;
	int 	myServiceInterface;
	String 	myServiceType;
	Object	myProxy;
	Object 	myService;
	
	public SensorServiceDescription(int id,
	        		String gatewayIp, 
					String gatewayPort,
					String deviceIp,
					String serviceInterface,
					String serviceType){
	    
	    myId				= 	id;
        myGatewayIp 		= 	gatewayIp;
		myGatewayPort 		= 	gatewayPort;
		myDeviceIp			= 	deviceIp;
		myServiceInterface	=	getInterfaceType( serviceInterface );
		myServiceType		=	serviceType;
		myProxy				= 	createProxy();
   }
	
	public String getIpAddress(){
	    return myDeviceIp;
	}
	
	private String createProxyFormatedDeviceIpAddress(){
	    return  myGatewayIp  + ":" +  myGatewayPort + "/http://"  + myDeviceIp ;
	}
	
	private int getInterfaceType( String interfacName ){   
	    if( interfacName.equals( "IGasSensor" )){
	        return INTERFACE_GAS_SENSOR;
	    }	    
	    if( interfacName.equals( "IMotionSensor" )){
	        return INTERFACE_MOTION_SENSOR;
	    }    
	    if( interfacName.equals( "IUvmSensor" )){
	        return INTERFACE_UVM_SENSOR;
	    }    
	    return INTERFACE_UNKNOWN;
	    
	}
	public boolean matchGateway( String gatewayIp, String gatewayPort){
	    return 	gatewayIp.equals( myGatewayIp ) &&
	    		gatewayPort.equals( myGatewayPort );
	    
	}
	public String key(){
	    return SensorMap.getKey( myGatewayIp, myGatewayPort, myDeviceIp);
	}

	public int getId(){
	    return myId;
	}
	
	public Object getProxy(){
	    return myProxy;
	}
	
	private Object createProxy(){
	    switch( myServiceInterface ){
	   		case INTERFACE_GAS_SENSOR:
	    	    return new IGenericSensorProxy(createProxyFormatedDeviceIpAddress()); 
	    	case INTERFACE_MOTION_SENSOR:
	    	    return new IMotionDetectorProxy(createProxyFormatedDeviceIpAddress()); 
	   		case INTERFACE_UVM_SENSOR:
	    	    return new IGasSensorUvmProxy(createProxyFormatedDeviceIpAddress()); 
	    }
	    return null;
	}
	
	public Object createService( 	String simulatorProxy, 
	        						String simulatorPort,
	        						String simulatorIp ){
	        switch(myServiceInterface){
	        	case INTERFACE_GAS_SENSOR:
			        return new GenericSensorSimImpl( myId,
        					simulatorProxy, 
        					simulatorPort,
        					simulatorIp);
	        	case INTERFACE_MOTION_SENSOR:
			        return new MotionSensorSimImpl( myId,
	    					simulatorProxy, 
	    					simulatorPort,
	    					simulatorIp);
		   		case INTERFACE_UVM_SENSOR:
				    return new GasSensorUvmImpl( myId,
							simulatorProxy, 
							simulatorPort,
							simulatorIp);     
	        }
	    return null;
	}
}