package Parsers.BIFv01;

import InterchangeFormat.*;

import java.util.Vector;
import java.util.Enumeration;




public class BIFv01 extends InterchangeFormat implements BIFv01Constants {
  InterchangeFormatBayesNet ifbn;

  public InterchangeFormatBayesNet get_ifbn() { return(ifbn); }


  void glob_undefined_text() throws ParseError {
         Token t;
         while (true) {
                 t = getToken(1);
                 if ((t.kind == 0) ||
                     (t.kind == NETWORK) ||
                     (t.kind == VARIABLE) ||
                     (t.kind == PROBABILITY))
                         break;
                 else
                         getNextToken();
           }
  }

  protected BIFv01TokenManager token_source;
  protected Token token = new Token();
  protected Token scanpos, lastpos;
  private int la;
  protected BIFv01 me;

  private int[] scan_hist_kind = new int[10000];
  private Token[] scan_hist_index = new Token[10000];
  int scan_hist_ptr = 0;

  public BIFv01(java.io.InputStream stream) {
    me = this;
    ASCII_CharStream input_stream = new ASCII_CharStream(stream, 1, 1);
    token_source = new BIFv01TokenManager(input_stream);
  }

  public void ReInit(java.io.InputStream stream) {
    ASCII_CharStream input_stream = new ASCII_CharStream(stream, 1, 1);
    token_source.ReInit(input_stream);
    token = new Token();
    scan_hist_ptr = 0;
  }

  public BIFv01(BIFv01TokenManager tm) {
    me = this;
    token_source = tm;
  }

  public void ReInit(BIFv01TokenManager tm) {
    token_source = tm;
    token = new Token();
    scan_hist_ptr = 0;
  }

  final private Token consume_token(int kind) throws ParseError {
    if (token.next != null) token = token.next;
    else token = token_source.getNextToken();
    scan_hist_kind[scan_hist_ptr] = kind;
    scan_hist_index[scan_hist_ptr++] = token;
    if (token.kind == kind) {
      scan_hist_ptr = 0;
      return token;
    }
    token_error_setup(0);
    throw new ParseError();
  }

  final private boolean scan_token(int kind) throws ParseError {
    if (scanpos == lastpos) {
      la--;
      if (scanpos.next == null) {
        lastpos = scanpos = scanpos.next = token_source.getNextToken();
      } else {
        lastpos = scanpos = scanpos.next;
      }
    } else {
      scanpos = scanpos.next;
    }
    scan_hist_kind[scan_hist_ptr] = kind;
    scan_hist_index[scan_hist_ptr++] = scanpos;
    return (scanpos.kind != kind);
  }

  final public Token getNextToken() throws ParseError {
    if (token.next != null) token = token.next;
    else token = token_source.getNextToken();
    return token;
  }

  final public Token getToken(int index) throws ParseError {
    scanpos = token;
    for (int i = 0; i < index; i++) {
      if (scanpos.next != null) scanpos = scanpos.next;
      else scanpos = scanpos.next = token_source.getNextToken();
    }
    return scanpos;
  }

  static final String add_escapes(String str) {
    String retval = "";
    char ch;
    for (int i = 0; i < str.length(); i++) {
      ch = str.charAt(i);
      if (ch == '\b') {
        retval += "\\b";
      } else if (ch == '\t') {
        retval += "\\t";
      } else if (ch == '\n') {
        retval += "\\n";
      } else if (ch == '\f') {
        retval += "\\f";
      } else if (ch == '\r') {
        retval += "\\r";
      } else if (ch == '\"') {
        retval += "\\\"";
      } else if (ch == '\'') {
        retval += "\\\'";
      } else if (ch == '\\') {
        retval += "\\\\";
      } else {
        retval += ch;
      }
    }
    return retval;
  }

  protected int error_line;
  protected int error_column;
  protected String error_string;
  protected String[] expected_tokens;

  protected void token_error() {
    System.out.println("");
    System.out.println("Parse error at line " + error_line + ", column " + error_column + ".  Encountered:");
    System.out.println("    \"" + add_escapes(error_string) + "\"");
    System.out.println("");
    if (expected_tokens.length == 1) {
      System.out.println("Was expecting:");
    } else {
      System.out.println("Was expecting one of:");
    }
    for (int i = 0; i < expected_tokens.length; i++) {
      System.out.println("    " + expected_tokens[i]);
    }
    System.out.println("");
  }

  final protected void token_error_setup(int stream_ptr) {
    Token tok;
    int[] tokenbuf = new int[100];
    int tokenbufptr = 0;
    int maxsize = 0;
    String[] exp = new String[1000];
    int exp_count = 0;
    for (int i = 0; i < scan_hist_ptr; i++) {
      if (i == 0 || scan_hist_index[i-1].next == scan_hist_index[i]) {
        tokenbuf[tokenbufptr++] = scan_hist_kind[i];
        if (tokenbufptr > maxsize) maxsize = tokenbufptr;
      } else {
        exp[exp_count] = "";
        for (int j = 0; j < tokenbufptr; j++) {
          exp[exp_count] += tokenImage[tokenbuf[j]] + " ";
        }
        if (tokenbuf[tokenbufptr-1] != 0) exp[exp_count] += "...";
        exp_count++;
        for (int j = 0; j < exp_count-1; j++) {
          if (exp[j].equals(exp[exp_count-1])) {
            exp_count--; break;
          }
        }
        tok = token;
        if (stream_ptr == 1) tok = tok.next;
        tokenbufptr = 1;
        while (tok != scan_hist_index[i]) {
          tok = tok.next;
          tokenbufptr++;
        }
        tokenbuf[tokenbufptr-1] = scan_hist_kind[i];
      }
    }
    exp[exp_count] = "";
    for (int j = 0; j < tokenbufptr; j++) {
      exp[exp_count] += tokenImage[tokenbuf[j]] + " ";
    }
    if (tokenbuf[tokenbufptr-1] != 0) exp[exp_count] += "...";
    exp_count++;
    for (int j = 0; j < exp_count-1; j++) {
      if (exp[j].equals(exp[exp_count-1])) {
        exp_count--; break;
      }
    }
    expected_tokens = new String[exp_count];
    for (int i = 0; i < exp_count; i++) {
      expected_tokens[i] = exp[i];
    }
    error_line = token.beginLine;
    error_column = token.beginColumn;
    error_string = "";
    tok = token;
    if (stream_ptr == 1) tok = tok.next;
    for (int i = 0; i < maxsize; i++) {
      if (tok.kind == 0) {
        error_string += " " + tokenImage[0];
        break;
      }
      error_string += " " + tok.image;
      tok = tok.next;
    }
    error_string = error_string.substring(1);
    me.token_error();
  }

  final public void CompilationUnit() throws ParseError {
InterchangeFormatProbabilityVariable pv;
InterchangeFormatProbabilityFunction upf;
           glob_undefined_text();
    NetworkDeclaration();
           glob_undefined_text();
    while (_Jack2_1(1)) {
      if (_Jack2_2(1)) {
        pv = VariableDeclaration();
           ifbn.add(pv); glob_undefined_text();
      } else if (_Jack2_3(1)) {
        upf = ProbabilityDeclaration();
           ifbn.add(upf); glob_undefined_text();
      } else {
        token_error_setup(1);
        throw new ParseError();
      }
    }
    consume_token(0);
  }

  final public void NetworkDeclaration() throws ParseError {
Token t;
Vector properties;
    consume_token(NETWORK);
    t = consume_token(WORD);
    properties = NetworkContent();
           ifbn = new InterchangeFormatBayesNet(t.image, properties);
  }

  final public Vector NetworkContent() throws ParseError {
Vector properties = new Vector();
String s;
    consume_token(22);
    while (_Jack2_4(1)) {
      s = Property();
           properties.addElement(s);
    }
    consume_token(23);
           return(properties);
  }

  final public InterchangeFormatProbabilityVariable VariableDeclaration() throws ParseError {
String s;
InterchangeFormatProbabilityVariable pv;
    consume_token(VARIABLE);
    s = ProbabilityVariableName();
    pv = VariableContent(s);
           return(pv);
  }

  final public InterchangeFormatProbabilityVariable VariableContent(String name) throws ParseError {
String s;
String values[] = null;
Vector properties = new Vector();
InterchangeFormatProbabilityVariable pv = new InterchangeFormatProbabilityVariable();
    consume_token(22);
    while (_Jack2_5(1)) {
      if (_Jack2_6(1)) {
        s = Property();
           properties.addElement(s);
      } else if (_Jack2_7(1)) {
        values = VariableDiscrete();
      } else {
        token_error_setup(1);
        throw new ParseError();
      }
    }
    consume_token(23);
           pv.set_name(name);
           pv.set_properties(properties);
           pv.set_values(values);
           return(pv);
  }

  final public String[] VariableDiscrete() throws ParseError {
String values[] = null;
    consume_token(VARIABLETYPE);
    consume_token(DISCRETE);
    consume_token(24);
    consume_token(NUMBER);
    consume_token(25);
    consume_token(22);
    values = VariableValuesList();
    consume_token(23);
    consume_token(26);
           return(values);
  }

  final public String[] VariableValuesList() throws ParseError {
int i;
String value;
String values[] = null;
Vector v = new Vector();
Enumeration e;
    value = ProbabilityVariableValue();
           v.addElement(value);
    while (_Jack2_8(1)) {
      value = ProbabilityVariableValue();
           v.addElement(value);
    }
           values = new String[v.size()];
           for (e=v.elements(), i=0; e.hasMoreElements(); i++)
                 values[i] = (String)(e.nextElement());
           return(values);
  }

  final public String ProbabilityVariableValue() throws ParseError {
Token t;
    if (_Jack2_9(1)) {
      t = consume_token(WORD);
    } else if (_Jack2_10(1)) {
      t = consume_token(NUMBER);
    } else {
      token_error_setup(1);
      throw new ParseError();
    }
           return(t.image);
  }

  final public InterchangeFormatProbabilityFunction ProbabilityDeclaration() throws ParseError {
String vs[];
InterchangeFormatProbabilityFunction upf = new InterchangeFormatProbabilityFunction();
    consume_token(PROBABILITY);
    ProbabilityVariablesList(upf);
    ProbabilityContent(upf);
           return(upf);
  }

  final public void ProbabilityVariablesList(InterchangeFormatProbabilityFunction upf) throws ParseError {
int i;
Enumeration e;
String variable_name;
int cond = -1;
String vs[];
Vector v_list = new Vector();
    consume_token(27);
    variable_name = ProbabilityVariableName();
    if (_Jack2_11(1)) {
      cond = ConditionalMark(v_list);
    }
           v_list.addElement(variable_name);
    while (_Jack2_12(1)) {
      variable_name = ProbabilityVariableName();
      if (_Jack2_13(1)) {
        cond = ConditionalMark(v_list);
      }
           v_list.addElement(variable_name);
    }
    consume_token(28);
           vs = new String[v_list.size()];
           for (e=v_list.elements(), i=0; e.hasMoreElements(); i++)
                 vs[i] = (String)(e.nextElement());
           upf.set_variables(vs);
           upf.set_conditional_index(cond);
  }

  final public int ConditionalMark(Vector v) throws ParseError {
    consume_token(29);
      return(v.size());
  }

  final public String ProbabilityVariableName() throws ParseError {
Token t;
    t = consume_token(WORD);
           return(t.image);
  }

  final public void ProbabilityContent(InterchangeFormatProbabilityFunction upf) throws ParseError {
String s = null;
Vector properties = new Vector();
InterchangeFormatProbabilityEntry e = null;
Vector entries = new Vector();
Vector defs = new Vector();
Vector tabs = new Vector();
double def[] = null;
double tab[] = null;
    consume_token(22);
    while (_Jack2_14(1)) {
      if (_Jack2_15(1)) {
        s = Property();
           properties.addElement(s);
      } else if (_Jack2_16(1)) {
        def = ProbabilityDefaultEntry();
      defs.addElement(def);
      } else if (_Jack2_17(1)) {
        e = ProbabilityEntry();
           entries.addElement(e);
      } else if (_Jack2_18(1)) {
        tab = ProbabilityTable();
      tabs.addElement(tab);
      } else {
        token_error_setup(1);
        throw new ParseError();
      }
    }
    consume_token(23);
           upf.set_properties(properties);
           upf.set_defaults(defs);
           upf.set_entries(entries);
           upf.set_tables(tabs);
  }

  final public InterchangeFormatProbabilityEntry ProbabilityEntry() throws ParseError {
String s[];
double d[];
    s = ProbabilityValuesList();
    d = FloatingPointList();
    consume_token(26);
           return( new InterchangeFormatProbabilityEntry(s, d) );
  }

  final public String[] ProbabilityValuesList() throws ParseError {
int i;
Enumeration e;
String variable_name;
String vs[];
Vector v_list = new Vector();
    consume_token(27);
    variable_name = ProbabilityVariableValue();
           v_list.addElement(variable_name);
    while (_Jack2_19(1)) {
      variable_name = ProbabilityVariableValue();
           v_list.addElement(variable_name);
    }
    consume_token(28);
           vs = new String[v_list.size()];
           for (e=v_list.elements(), i=0; e.hasMoreElements(); i++)
                 vs[i] = (String)(e.nextElement());
           return(vs);
  }

  final public double[] ProbabilityDefaultEntry() throws ParseError {
double d[];
    consume_token(DEFAULTVALUE);
    d = FloatingPointList();
    consume_token(26);
           return(d);
  }

  final public double[] ProbabilityTable() throws ParseError {
double d[];
    consume_token(TABLEVALUES);
    d = FloatingPointList();
    consume_token(26);
           return(d);
  }

  final public double[] FloatingPointList() throws ParseError {
int i;
Double d;
double ds[];
Vector d_list = new Vector();
Enumeration e;
    d = FloatingPointNumber();
           d_list.addElement(d);
    while (_Jack2_20(1)) {
      d = FloatingPointNumber();
           d_list.addElement(d);
    }
           ds = new double[d_list.size()];
           for (e=d_list.elements(), i=0; e.hasMoreElements(); i++) {
                 d = (Double)(e.nextElement());
                 ds[i] = d.doubleValue();
           }
           return(ds);
  }

  final public Double FloatingPointNumber() throws ParseError {
Token t;
    t = consume_token(NUMBER);
           return( Double.valueOf(t.image) );
  }

  final public String Property() throws ParseError {
int k;
Token t;
String s;
    t = consume_token(PROPERTYSTRING);
           s = t.image;
           k = s.indexOf(' ');
           return(s.substring(k, s.length() - 1));
  }

  final private boolean _Jack2_1(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_1();
  }

  final private boolean _Jack2_2(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_2();
  }

  final private boolean _Jack2_3(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_3();
  }

  final private boolean _Jack2_4(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_4();
  }

  final private boolean _Jack2_5(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_5();
  }

  final private boolean _Jack2_6(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_6();
  }

  final private boolean _Jack2_7(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_7();
  }

  final private boolean _Jack2_8(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_8();
  }

  final private boolean _Jack2_9(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_9();
  }

  final private boolean _Jack2_10(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_10();
  }

  final private boolean _Jack2_11(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_11();
  }

  final private boolean _Jack2_12(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_12();
  }

  final private boolean _Jack2_13(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_13();
  }

  final private boolean _Jack2_14(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_14();
  }

  final private boolean _Jack2_15(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_15();
  }

  final private boolean _Jack2_16(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_16();
  }

  final private boolean _Jack2_17(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_17();
  }

  final private boolean _Jack2_18(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_18();
  }

  final private boolean _Jack2_19(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_19();
  }

  final private boolean _Jack2_20(int xla) throws ParseError {
    la = xla; lastpos = scanpos = token;
    return !_Jack3_20();
  }

  final private boolean _Jack3_1() throws ParseError {
    Token xsp;
    xsp = scanpos;
    if (_Jack3_2()) {
    scanpos = xsp;
    if (_Jack3_3()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    } else if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_2() throws ParseError {
    if (_Jack3VariableDeclaration()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_3() throws ParseError {
    if (_Jack3ProbabilityDeclaration()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_4() throws ParseError {
    if (_Jack3Property()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_5() throws ParseError {
    Token xsp;
    xsp = scanpos;
    if (_Jack3_6()) {
    scanpos = xsp;
    if (_Jack3_7()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    } else if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_6() throws ParseError {
    if (_Jack3Property()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_7() throws ParseError {
    if (_Jack3VariableDiscrete()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_8() throws ParseError {
    if (_Jack3ProbabilityVariableValue()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_9() throws ParseError {
    if (scan_token(WORD)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_10() throws ParseError {
    if (scan_token(NUMBER)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_11() throws ParseError {
    if (_Jack3ConditionalMark()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_12() throws ParseError {
    if (_Jack3ProbabilityVariableName()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    Token xsp;
    xsp = scanpos;
    if (_Jack3_13()) scanpos = xsp;
    else if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_13() throws ParseError {
    if (_Jack3ConditionalMark()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_14() throws ParseError {
    Token xsp;
    xsp = scanpos;
    if (_Jack3_15()) {
    scanpos = xsp;
    if (_Jack3_16()) {
    scanpos = xsp;
    if (_Jack3_17()) {
    scanpos = xsp;
    if (_Jack3_18()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    } else if (la == 0 && scanpos == lastpos) return false;
    } else if (la == 0 && scanpos == lastpos) return false;
    } else if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_15() throws ParseError {
    if (_Jack3Property()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_16() throws ParseError {
    if (_Jack3ProbabilityDefaultEntry()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_17() throws ParseError {
    if (_Jack3ProbabilityEntry()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_18() throws ParseError {
    if (_Jack3ProbabilityTable()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_19() throws ParseError {
    if (_Jack3ProbabilityVariableValue()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3_20() throws ParseError {
    if (_Jack3FloatingPointNumber()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3VariableDeclaration() throws ParseError {
    if (scan_token(VARIABLE)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (_Jack3ProbabilityVariableName()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (_Jack3VariableContent()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3ProbabilityDeclaration() throws ParseError {
    if (scan_token(PROBABILITY)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (_Jack3ProbabilityVariablesList()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (_Jack3ProbabilityContent()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3Property() throws ParseError {
    if (scan_token(PROPERTYSTRING)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3VariableDiscrete() throws ParseError {
    if (scan_token(VARIABLETYPE)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (scan_token(DISCRETE)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (scan_token(24)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (scan_token(NUMBER)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (scan_token(25)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (scan_token(22)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (_Jack3VariableValuesList()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (scan_token(23)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (scan_token(26)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3ProbabilityVariableValue() throws ParseError {
    Token xsp;
    xsp = scanpos;
    if (_Jack3_9()) {
    scanpos = xsp;
    if (_Jack3_10()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    } else if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3ConditionalMark() throws ParseError {
    if (scan_token(29)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3ProbabilityVariableName() throws ParseError {
    if (scan_token(WORD)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3ProbabilityDefaultEntry() throws ParseError {
    if (scan_token(DEFAULTVALUE)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (_Jack3FloatingPointList()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (scan_token(26)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3ProbabilityEntry() throws ParseError {
    if (_Jack3ProbabilityValuesList()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (_Jack3FloatingPointList()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (scan_token(26)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3ProbabilityTable() throws ParseError {
    if (scan_token(TABLEVALUES)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (_Jack3FloatingPointList()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (scan_token(26)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3FloatingPointNumber() throws ParseError {
    if (scan_token(NUMBER)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3VariableContent() throws ParseError {
    if (scan_token(22)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    Token xsp;
    while (true) {
      xsp = scanpos;
      if (_Jack3_5()) { scanpos = xsp; break; }
      if (la == 0 && scanpos == lastpos) return false;
    }
    if (scan_token(23)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3ProbabilityVariablesList() throws ParseError {
    if (scan_token(27)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (_Jack3ProbabilityVariableName()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    Token xsp;
    xsp = scanpos;
    if (_Jack3_11()) scanpos = xsp;
    else if (la == 0 && scanpos == lastpos) return false;
    while (true) {
      xsp = scanpos;
      if (_Jack3_12()) { scanpos = xsp; break; }
      if (la == 0 && scanpos == lastpos) return false;
    }
    if (scan_token(28)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3ProbabilityContent() throws ParseError {
    if (scan_token(22)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    Token xsp;
    while (true) {
      xsp = scanpos;
      if (_Jack3_14()) { scanpos = xsp; break; }
      if (la == 0 && scanpos == lastpos) return false;
    }
    if (scan_token(23)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

  final private boolean _Jack3VariableValuesList() throws ParseError {
    if (_Jack3ProbabilityVariableValue()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    Token xsp;
    while (true) {
      xsp = scanpos;
      if (_Jack3_8()) { scanpos = xsp; break; }
      if (la == 0 && scanpos == lastpos) return false;
    }
    return false;
  }

  final private boolean _Jack3FloatingPointList() throws ParseError {
    if (_Jack3FloatingPointNumber()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    Token xsp;
    while (true) {
      xsp = scanpos;
      if (_Jack3_20()) { scanpos = xsp; break; }
      if (la == 0 && scanpos == lastpos) return false;
    }
    return false;
  }

  final private boolean _Jack3ProbabilityValuesList() throws ParseError {
    if (scan_token(27)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    if (_Jack3ProbabilityVariableValue()) return true;
    if (la == 0 && scanpos == lastpos) return false;
    Token xsp;
    while (true) {
      xsp = scanpos;
      if (_Jack3_19()) { scanpos = xsp; break; }
      if (la == 0 && scanpos == lastpos) return false;
    }
    if (scan_token(28)) return true;
    if (la == 0 && scanpos == lastpos) return false;
    return false;
  }

}
