/*
 * TotalVariationQBProbabilityFunction.java
 * Copyright 1996, 1997 Fabio Cozman
 * <fgcozman@cs.cmu.edu><http://www.cs.cmu.edu/~fgcozman/home.html>
 * JavaBayes information and source code:
 * <http://www.cs.cmu.edu/~fgcozman/Research/JavaBayes/Home/>
 *
 * The BayesianNetworks package is free software; you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation, provided
 * that this notice and the name of the author appear in all copies.
 * If you're using the software, please notify fgcozman@cs.cmu.edu so
 * that you can receive updates and patches.
 * The BayesianNetworks package is distributed "as is", in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * You should have received a copy of the source code for the package;
 * if not, look for the JavaBayes distribution, which is freely available.
 * You should have received a copy of the GNU General Public License
 * along with the BayesianNetworks package. If not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package QuasiBayesianNetworks;

import BayesianNetworks.*;

/*******************************************************************/

public class TotalVariationQBProbabilityFunction
        extends TwoMonotoneCapacity {

   private double epsilon;

  /***************************************************************/
  /* Constructor for an TotalVariationQBProbabilityFunction      */
  /* ProbabilityFunction object and given epsilon                */
  /***************************************************************/
  public TotalVariationQBProbabilityFunction(ProbabilityFunction pf,
                                      double eps) {
     super(pf);
     epsilon = eps;
     if ((epsilon < 0.0) || (epsilon > 1.0)) epsilon = 0.0;
  }

  /***************************************************************/
  /* Obtain the lower probability of an event given the base     */
  /* probability for the event.                                  */
  /***************************************************************/
  public double get_lower_probability_from_base(double p) {
    return( Math.max( p - epsilon, 0.0 ) );
  }

  /***************************************************************/
  /* Obtain the upper probability of an event given the base     */
  /* probability for the event.                                  */
  /***************************************************************/
  public double get_upper_probability_from_base(double p) {
    return( Math.min( p + epsilon, 1.0 ) );
  }

  /***************************************************************/
  /* Get a base probability value for an atom.                   */
  /***************************************************************/
  public double get_atom_probability(int index) {
    return( values[index] );
  }

}